import datetime
from typing import List, Optional, Union, type_check_only
from typing_extensions import TypedDict

from .channel import GroupChannel
from .enums import VoiceRegion
from .member import VoiceState
from .message import Message
from .user import ClientUser, User

# TODO: remove this comment when a new version of black comes out
@type_check_only
class _BaseVoiceStateDict(TypedDict):
    channel_id: Optional[int]
    user_id: int
    session_id: str
    deaf: bool
    mute: bool
    self_deaf: bool
    self_mute: bool
    suppress: bool

@type_check_only
class _VoiceStateDict(_BaseVoiceStateDict, total=False):
    guild_id: int

class CallMessage:
    ended_timestamp: Optional[datetime.datetime]
    participants: Optional[List[User]]
    message: Message
    def __init__(
        self,
        message: Message,
        *,
        ended_timestamp: Optional[str] = ...,
        participants: List[User],
    ) -> None: ...
    @property
    def call_ended(self) -> bool: ...
    @property
    def channel(self) -> GroupChannel: ...
    @property
    def duration(self) -> datetime.timedelta: ...

class GroupCall:
    call: CallMessage
    unavailable: Optional[bool]
    ringing: List[User]
    region: VoiceRegion
    def __init__(
        self,
        *,
        call: CallMessage,
        unavailable: bool,
        voice_states: List[_VoiceStateDict] = ...,
        region: VoiceRegion,
        ringing: List[int] = ...,
    ) -> None: ...
    @property
    def connected(self) -> List[Union[User, ClientUser]]: ...
    @property
    def channel(self) -> GroupChannel: ...
    def voice_state_for(
        self, user: Union[User, ClientUser]
    ) -> Optional[VoiceState]: ...



from .enums import RelationshipType
from .user import User

class Relationship:
    user: User
    type: RelationshipType
    async def delete(self) -> None: ...
    async def accept(self) -> None: ...



import datetime
from typing import Optional

from .enums import ExpireBehaviour
from .guild import Guild
from .role import Role
from .user import User

class IntegrationAccount:
    id: int
    name: str

class Integration:
    id: int
    name: str
    guild: Guild
    type: str
    enabled: bool
    syncing: bool
    role: Role
    enable_emoticons: Optional[bool]
    expire_behaviour: ExpireBehaviour
    expire_behavior: ExpireBehaviour
    expire_grace_period: int
    user: User
    account: IntegrationAccount
    synced_at: datetime.datetime
    async def edit(
        self,
        *,
        expire_behaviour: ExpireBehaviour = ...,
        expire_behavior: ExpireBehaviour = ...,
        expire_grace_period: int = ...,
        enable_emoticons: bool = ...,
    ) -> None: ...
    async def sync(self) -> None: ...
    async def delete(self) -> None: ...



from typing import Any

class EqualityComparable:
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...

class Hashable(EqualityComparable):
    def __hash__(self) -> int: ...



import asyncio
import datetime
from typing import (
    Any,
    Awaitable,
    Callable,
    Generator,
    Generic,
    Optional,
    Type,
    TypeVar,
    Union,
    overload,
    type_check_only,
)
from typing_extensions import Protocol

_T = TypeVar('_T')
_L = TypeVar('_L', bound=Loop[Any])
_CoroType = Callable[..., Union[Awaitable[_T], Generator[Any, None, _T]]]
_C = TypeVar('_C', bound=Callable[..., Awaitable[Any]])

# TODO: remove this comment when a new version of black comes out
@type_check_only
class _LoopCallback(Protocol):
    async def __call__(self) -> Any: ...

@type_check_only
class _ErrorCallback(Protocol):
    async def __call__(self, __exc: BaseException) -> Any: ...

class Loop(Generic[_T]):
    coro: _CoroType[_T]
    seconds: float
    hours: float
    minutes: float
    reconnect: bool
    loop: asyncio.AbstractEventLoop
    count: Optional[int]
    def __init__(
        self,
        coro: _CoroType[_T],
        seconds: float,
        hours: float,
        minutes: float,
        count: Optional[int],
        reconnect: bool,
        loop: Optional[asyncio.AbstractEventLoop],
    ) -> None: ...
    @overload
    def __get__(self: _L, obj: None, objtype: Any) -> _L: ...
    @overload
    def __get__(self, obj: Any, objtype: Any) -> Loop[_T]: ...
    @property
    def current_loop(self) -> int: ...
    @property
    def next_iteration(self) -> Optional[datetime.datetime]: ...
    async def __call__(self, *args: Any, **kwargs: Any) -> _T: ...
    def start(self, *args: Any, **kwargs: Any) -> asyncio.Task[_T]: ...
    def stop(self) -> None: ...
    def cancel(self) -> None: ...
    def restart(self, *args: Any, **kwargs: Any) -> None: ...
    def add_exception_type(self, *exceptions: Type[BaseException]) -> None: ...
    def clear_exception_types(self) -> None: ...
    def remove_exception_type(self, *exceptions: Type[BaseException]) -> bool: ...
    def get_task(self) -> asyncio.Task[_T]: ...
    def is_being_cancelled(self) -> bool: ...
    def failed(self) -> bool: ...
    def is_running(self) -> bool: ...
    def before_loop(self, coro: _LoopCallback) -> _LoopCallback: ...
    def after_loop(self, coro: _LoopCallback) -> _LoopCallback: ...
    def error(self, coro: _ErrorCallback) -> _ErrorCallback: ...
    def change_interval(
        self, *, seconds: float = ..., minutes: float = ..., hours: float = ...
    ) -> None: ...

def loop(
    *,
    seconds: float = ...,
    minutes: float = ...,
    hours: float = ...,
    count: Optional[int] = ...,
    reconnect: bool = ...,
    loop: Optional[asyncio.AbstractEventLoop] = ...,
) -> Callable[[_CoroType[_T]], Loop[_T]]: ...



import datetime
from typing import ClassVar, List, Optional, Union

from .asset import _VALID_ANIMATED_ICON_FORMATS, _VALID_STATIC_ICON_FORMATS, Asset
from .channel import StoreChannel, TextChannel, VoiceChannel
from .enums import ChannelType, VerificationLevel
from .guild import Guild
from .mixins import Hashable
from .object import Object
from .user import User

class PartialInviteChannel:
    id: int
    name: str
    type: ChannelType
    @property
    def mention(self) -> str: ...
    @property
    def created_at(self) -> datetime.datetime: ...

class PartialInviteGuild:
    features: List[str]
    icon: Optional[str]
    banner: Optional[str]
    id: int
    name: str
    splash: Optional[str]
    verification_level: VerificationLevel
    description: Optional[str]
    @property
    def created_at(self) -> datetime.datetime: ...
    @property
    def icon_url(self) -> Asset: ...
    def is_icon_animated(self) -> bool: ...
    def icon_url_as(
        self,
        *,
        format: Optional[_VALID_ANIMATED_ICON_FORMATS] = ...,
        static_format: _VALID_STATIC_ICON_FORMATS = ...,
        size: int = ...,
    ) -> Asset: ...
    @property
    def banner_url(self) -> Asset: ...
    def banner_url_as(
        self, *, format: _VALID_STATIC_ICON_FORMATS = ..., size: int = ...
    ) -> Asset: ...
    @property
    def splash_url(self) -> Asset: ...
    def splash_url_as(
        self, *, format: _VALID_STATIC_ICON_FORMATS = ..., size: int = ...
    ) -> Asset: ...

class Invite(Hashable):
    BASE: ClassVar[str]

    max_age: int
    code: str
    guild: Optional[Union[Guild, Object, PartialInviteGuild]]
    revoked: bool
    created_at: datetime.datetime
    temporary: bool
    uses: int
    max_uses: int
    inviter: User
    approximate_member_count: Optional[int]
    approximate_presence_count: Optional[int]
    channel: Union[
        TextChannel, VoiceChannel, StoreChannel, Object, PartialInviteChannel
    ]
    def __hash__(self) -> int: ...
    @property
    def id(self) -> str: ...
    @property
    def url(self) -> str: ...
    async def delete(self, *, reason: Optional[str] = ...) -> None: ...



from typing import Any, ClassVar, List, Optional, Pattern

import discord

from .bot import Bot
from .context import Context

class Converter:
    async def convert(self, ctx: Context, argument: str) -> Any: ...

class IDConverter(Converter): ...

class MemberConverter(IDConverter):
    async def query_member_named(
        self, guild: discord.Guild, argument: str
    ) -> Optional[discord.Member]: ...
    async def query_member_by_id(
        self, bot: Bot[Context], guild: discord.Guild, user_id: int
    ) -> Optional[discord.Member]: ...
    async def convert(self, ctx: Context, argument: str) -> discord.Member: ...

class UserConverter(IDConverter):
    async def convert(self, ctx: Context, argument: str) -> discord.User: ...

class PartialMessageConverter(Converter):
    async def convert(self, ctx: Context, argument: str) -> discord.PartialMessage: ...

class MessageConverter(PartialMessageConverter):
    async def convert(  # type: ignore[override]
        self, ctx: Context, argument: str
    ) -> discord.Message: ...

class TextChannelConverter(IDConverter):
    async def convert(self, ctx: Context, argument: str) -> discord.TextChannel: ...

class VoiceChannelConverter(IDConverter):
    async def convert(self, ctx: Context, argument: str) -> discord.VoiceChannel: ...

class StageChannelConverter(IDConverter):
    async def convert(self, ctx: Context, argument: str) -> discord.StageChannel: ...

class CategoryChannelConverter(IDConverter):
    async def convert(self, ctx: Context, argument: str) -> discord.CategoryChannel: ...

class StoreChannelConverter(IDConverter):
    async def convert(self, ctx: Context, argument: str) -> discord.StoreChannel: ...

class ColourConverter(Converter):
    RGB_REGEX: ClassVar[Pattern[str]]
    def parse_hex_number(self, argument: str) -> discord.Colour: ...
    def parse_rgb_number(self, argument: str, number: str) -> int: ...
    def parse_rgb(
        self, argument: str, *, regex: Pattern[str] = ...
    ) -> discord.Colour: ...
    async def convert(self, ctx: Context, argument: str) -> discord.Colour: ...

ColorConverter = ColourConverter

class RoleConverter(IDConverter):
    async def convert(self, ctx: Context, argument: str) -> discord.Role: ...

class GameConverter(Converter):
    async def convert(self, ctx: Context, argument: str) -> discord.Game: ...

class InviteConverter(Converter):
    async def convert(self, ctx: Context, argument: str) -> discord.Invite: ...

class GuildConverter(IDConverter):
    async def convert(self, ctx: Context, argument: str) -> discord.Guild: ...

class EmojiConverter(IDConverter):
    async def convert(self, ctx: Context, argument: str) -> discord.Emoji: ...

class PartialEmojiConverter(Converter):
    async def convert(self, ctx: Context, argument: str) -> discord.PartialEmoji: ...

class clean_content(Converter):
    def __init__(
        self,
        *,
        fix_channel_mentions: bool = ...,
        use_nicknames: bool = ...,
        escape_markdown: bool = ...,
        remove_markdown: bool = ...,
    ) -> None: ...
    async def convert(self, ctx: Context, argument: str) -> str: ...

Greedy = List



import asyncio
from typing import Any, Callable, ClassVar, Dict, Optional, Tuple

from .abc import Connectable
from .channel import VoiceChannel
from .client import Client
from .gateway import DiscordVoiceWebSocket
from .guild import Guild
from .player import AudioSource
from .user import ClientUser

class VoiceProtocol:
    client: Client
    channel: Connectable
    def __init__(self, client: Client, channel: Connectable) -> None: ...
    async def on_voice_state_update(self, data: Dict[str, Any]) -> None: ...
    async def on_voice_server_update(self, data: Dict[str, Any]) -> None: ...
    async def connect(self, *, timeout: float, reconnect: bool) -> None: ...
    async def disconnect(self, *, force: bool) -> None: ...
    def cleanup(self) -> None: ...

class VoiceClient(VoiceProtocol):
    warn_nacl: ClassVar[bool]
    supported_modes: ClassVar[Tuple[str, ...]]

    session_id: str
    token: str
    endpoint: str
    channel: Connectable
    loop: asyncio.AbstractEventLoop
    source: Optional[AudioSource]
    @property
    def guild(self) -> Optional[Guild]: ...
    @property
    def user(self) -> ClientUser: ...
    def checked_add(self, attr: str, value: int, limit: int) -> None: ...
    async def on_voice_state_update(self, data: Dict[str, Any]) -> None: ...
    async def on_voice_server_update(self, data: Dict[str, Any]) -> None: ...
    async def voice_connect(self) -> None: ...
    async def voice_disconnect(self) -> None: ...
    def prepare_handshake(self) -> None: ...
    def finish_handshake(self) -> None: ...
    async def connect_websocket(self) -> DiscordVoiceWebSocket: ...
    async def connect(self, *, timeout: float, reconnect: bool) -> None: ...
    async def potential_reconnect(self) -> bool: ...
    @property
    def latency(self) -> float: ...
    @property
    def average_latency(self) -> float: ...
    async def poll_voice_ws(self, reconnect: bool) -> None: ...
    async def disconnect(self, *, force: bool = ...) -> None: ...
    async def move_to(self, channel: VoiceChannel) -> None: ...
    def is_connected(self) -> bool: ...
    def play(
        self,
        source: AudioSource,
        *,
        after: Optional[Callable[[Optional[Exception]], None]] = ...,
    ) -> None: ...
    def is_playing(self) -> bool: ...
    def is_paused(self) -> bool: ...
    def stop(self) -> None: ...
    def pause(self) -> None: ...
    def resume(self) -> None: ...
    def send_audio_packet(self, data: bytes, *, encode: bool = ...) -> None: ...



from inspect import Parameter
from typing import Any, List, Optional, Tuple, Union

from discord import (
    CategoryChannel,
    DMChannel,
    GroupChannel,
    Permissions,
    StoreChannel,
    TextChannel,
    VoiceChannel,
    abc,
)
from discord.errors import ClientException, DiscordException

from .cooldowns import BucketType, Cooldown

class CommandError(DiscordException):
    def __init__(self, message: Optional[str] = ..., *args: Any) -> None: ...

class ConversionError(DiscordException):
    converter: Any
    original: Exception
    def __init__(self, converter: Any, original: Exception) -> None: ...

class UserInputError(CommandError): ...
class CommandNotFound(CommandError): ...

class MissingRequiredArgument(UserInputError):
    param: Parameter
    def __init__(self, param: Parameter) -> None: ...

class TooManyArguments(UserInputError): ...
class BadArgument(UserInputError): ...
class CheckFailure(CommandError): ...
class CheckAnyFailure(CheckFailure): ...
class PrivateMessageOnly(CheckFailure): ...

class NoPrivateMessage(CheckFailure):
    def __init__(self, message: Optional[str] = ...) -> None: ...

class NotOwner(CheckFailure): ...

class MemberNotFound(BadArgument):
    argument: str

class UserNotFound(BadArgument):
    argument: str

class GuildNotFound(BadArgument):
    argument: str

class MessageNotFound(BadArgument):
    argument: str

class ChannelNotReadable(BadArgument):
    argument: Union[
        TextChannel,
        VoiceChannel,
        CategoryChannel,
        StoreChannel,
        DMChannel,
        GroupChannel,
    ]

class ChannelNotFound(BadArgument):
    argument: str

class BadColourArgument(BadArgument):
    argument: str

BadColorArgument = BadColourArgument

class RoleNotFound(BadArgument):
    argument: str

class BadInviteArgument(BadArgument): ...

class EmojiNotFound(BadArgument):
    argument: str

class PartialEmojiConversionFailure(BadArgument):
    argument: str

class BadBoolArgument(BadArgument):
    argument: str

class DisabledCommand(CommandError): ...

class CommandInvokeError(CommandError):
    original: Exception
    def __init__(self, e: Exception) -> None: ...

class CommandOnCooldown(CommandError):
    cooldown: Cooldown
    retry_after: float
    def __init__(self, cooldown: Cooldown, retry_after: float) -> None: ...

class MaxConcurrencyReached(CommandError):
    number: int
    per: BucketType
    def __init__(self, number: int, per: BucketType) -> None: ...

class MissingRole(CheckFailure):
    missing_role: Union[str, int]
    def __init__(self, missing_role: Union[str, int]) -> None: ...

class BotMissingRole(CheckFailure):
    missing_role: Union[str, int]
    def __init__(self, missing_role: Union[str, int]) -> None: ...

class MissingAnyRole(CheckFailure):
    missing_roles: List[Union[str, int]]
    def __init__(self, missing_roles: List[Union[str, int]]) -> None: ...

class BotMissingAnyRole(CheckFailure):
    missing_roles: List[Union[str, int]]
    def __init__(self, missing_roles: List[Union[str, int]]) -> None: ...

class NSFWChannelRequired(CheckFailure):
    channel: abc.GuildChannel
    def __init__(self, channel: abc.GuildChannel) -> None: ...

class MissingPermissions(CheckFailure):
    missing_perms: List[Permissions]
    def __init__(self, missing_perms: List[Permissions], *args: Any) -> None: ...

class BotMissingPermissions(CheckFailure):
    missing_perms: List[Permissions]
    def __init__(self, missing_perms: List[Permissions], *args: Any) -> None: ...

class BadUnionArgument(UserInputError):
    param: Parameter
    converters: Tuple[Any, ...]
    errors: List[CommandError]
    def __init__(
        self, param: Parameter, converters: Tuple[Any, ...], errors: List[CommandError]
    ) -> None: ...

class ArgumentParsingError(UserInputError): ...

class UnexpectedQuoteError(ArgumentParsingError):
    quote: str
    def __init__(self, quote: str) -> None: ...

class InvalidEndOfQuotedStringError(ArgumentParsingError):
    char: str
    def __init__(self, char: str) -> None: ...

class ExpectedClosingQuoteError(ArgumentParsingError):
    close_quote: str
    def __init__(self, close_quote: str) -> None: ...

class ExtensionError(DiscordException):
    name: str

class ExtensionAlreadyLoaded(ExtensionError):
    def __init__(self, name: str) -> None: ...

class ExtensionNotLoaded(ExtensionError):
    def __init__(self, name: str) -> None: ...

class NoEntryPointError(ExtensionError):
    def __init__(self, name: str) -> None: ...

class ExtensionFailed(ExtensionError):
    original: Exception
    def __init__(self, name: str, original: Exception) -> None: ...

class ExtensionNotFound(ExtensionError):
    original: ImportError
    def __init__(self, name: str, original: Optional[ImportError] = ...) -> None: ...

class CommandRegistrationError(ClientException):
    alias_conflict: bool
    def __init__(self, name: str, *, alias_conflict: bool = ...) -> None: ...



from typing import (
    Any,
    Callable,
    Coroutine,
    Generic,
    Iterator,
    List,
    Optional,
    Tuple,
    TypeVar,
    Union,
)

from .context import Context
from .core import Command, Group

_CT = TypeVar('_CT', bound=Context)
_L = TypeVar('_L', bound=Callable[..., Coroutine[Any, Any, Any]])

class CogMeta(type):
    @classmethod
    def qualified_name(cls) -> str: ...

class Cog(Generic[_CT], metaclass=CogMeta):
    __cog_commands__: Any
    def get_commands(self) -> List[Command[_CT]]: ...
    @property
    def qualified_name(self) -> str: ...
    @property
    def description(self) -> str: ...
    def walk_commands(self) -> Iterator[Union[Command[_CT], Group[_CT]]]: ...
    def get_listeners(self) -> List[Tuple[str, Callable[..., Any]]]: ...
    @classmethod
    def listener(cls, name: Optional[str] = ...) -> Callable[[_L], _L]: ...
    def has_error_handler(self) -> bool: ...
    def cog_unload(self) -> None: ...
    def bot_check_once(self, ctx: _CT) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    def bot_check(self, ctx: _CT) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    def cog_check(self, ctx: _CT) -> Union[bool, Coroutine[Any, Any, bool]]: ...
    async def cog_command_error(self, ctx: _CT, error: Any) -> None: ...
    async def cog_before_invoke(self, ctx: _CT) -> None: ...
    async def cog_after_invoke(self, ctx: _CT) -> None: ...



from typing import Optional

class StringView:
    def __init__(self, buffer: str) -> None: ...
    @property
    def current(self) -> Optional[str]: ...
    @property
    def eof(self) -> bool: ...
    def undo(self) -> None: ...
    def skip_ws(self) -> bool: ...
    def skip_string(self, string: str) -> bool: ...
    def read_rest(self) -> str: ...
    def read(self, n: int) -> str: ...
    def get(self) -> str: ...
    def get_word(self) -> str: ...
    def get_quoted_word(self) -> Optional[str]: ...



import datetime
from typing import Any, List, Optional, Union, overload

from .colour import Colour
from .guild import Guild
from .member import Member
from .mixins import Hashable
from .permissions import Permissions

class RoleTags:
    bot_id: Optional[int]
    integration_id: Optional[int]
    def is_bot_managed(self) -> bool: ...
    def is_premium_subscriber(self) -> bool: ...
    def is_integration(self) -> bool: ...

class Role(Hashable):
    id: int
    name: str
    guild: Guild
    hoist: bool
    position: int
    managed: bool
    mentionable: bool
    tags: Optional[RoleTags]
    def __lt__(self, other: Any) -> bool: ...
    def __le__(self, other: Any) -> bool: ...
    def __gt__(self, other: Any) -> bool: ...
    def __ge__(self, other: Any) -> bool: ...
    def is_default(self) -> bool: ...
    def is_bot_managed(self) -> bool: ...
    def is_premium_subscriber(self) -> bool: ...
    def is_integration(self) -> bool: ...
    @property
    def permissions(self) -> Permissions: ...
    @property
    def colour(self) -> Colour: ...
    @property
    def color(self) -> Colour: ...
    @property
    def created_at(self) -> datetime.datetime: ...
    @property
    def mention(self) -> str: ...
    @property
    def members(self) -> List[Member]: ...
    @overload
    async def edit(
        self,
        *,
        name: str = ...,
        permissions: Permissions = ...,
        colour: Union[Colour, int] = ...,
        hoist: bool = ...,
        mentionable: bool = ...,
        position: int = ...,
        reason: Optional[str] = ...,
    ) -> None: ...
    @overload
    async def edit(
        self,
        *,
        name: str = ...,
        permissions: Permissions = ...,
        color: Union[Colour, int] = ...,
        hoist: bool = ...,
        mentionable: bool = ...,
        position: int = ...,
        reason: Optional[str] = ...,
    ) -> None: ...
    async def delete(self, *, reason: Optional[str] = ...) -> None: ...



import asyncio
from typing import Any, ClassVar, List, Mapping, Optional, Tuple, Union, overload

import aiohttp

from .activity import BaseActivity
from .client import Client
from .enums import Status
from .flags import Intents, MemberCacheFlags
from .guild import Guild
from .mentions import AllowedMentions

class EventType:
    close: ClassVar[int]
    reconnect: ClassVar[int]
    resume: ClassVar[int]
    identify: ClassVar[int]
    terminate: ClassVar[int]
    clean_close: ClassVar[int]

class Shard:
    @property
    def id(self) -> int: ...
    def launch(self) -> None: ...
    async def close(self) -> None: ...
    async def disconnect(self) -> None: ...
    async def worker(self) -> None: ...
    async def reidentify(self, exc: BaseException) -> None: ...
    async def reconnect(self) -> None: ...

class ShardInfo:
    id: int
    shard_count: Optional[int]
    def is_closed(self) -> bool: ...
    async def disconnect(self) -> None: ...
    async def reconnect(self) -> None: ...
    async def connect(self) -> None: ...
    @property
    def latency(self) -> float: ...
    def is_ws_ratelimited(self) -> bool: ...

class AutoShardedClient(Client):
    shard_ids: Optional[Union[List[int], Tuple[int]]]
    @overload
    def __init__(
        self,
        *args: Any,
        shard_ids: Union[List[int], Tuple[int]],
        shard_count: int,
        max_messages: Optional[int] = ...,
        loop: Optional[asyncio.AbstractEventLoop] = ...,
        connector: aiohttp.BaseConnector = ...,
        proxy: Optional[str] = ...,
        proxy_auth: Optional[aiohttp.BasicAuth] = ...,
        intents: Optional[Intents] = ...,
        member_cache_flags: Optional[MemberCacheFlags] = ...,
        fetch_offline_members: bool = ...,
        chunk_guilds_at_startup: bool = ...,
        status: Optional[Status] = ...,
        activity: Optional[BaseActivity] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
        heartbeat_timeout: float = ...,
        guild_ready_timeout: float = ...,
        guild_subscriptions: bool = ...,
        assume_unsync_clock: bool = ...,
    ) -> None: ...
    @overload
    def __init__(
        self,
        *args: Any,
        shard_ids: None = ...,
        shard_count: Optional[int] = ...,
        max_messages: Optional[int] = ...,
        loop: Optional[asyncio.AbstractEventLoop] = ...,
        connector: aiohttp.BaseConnector = ...,
        proxy: Optional[str] = ...,
        proxy_auth: Optional[aiohttp.BasicAuth] = ...,
        intents: Optional[Intents] = ...,
        member_cache_flags: Optional[MemberCacheFlags] = ...,
        fetch_offline_members: bool = ...,
        chunk_guilds_at_startup: bool = ...,
        status: Optional[Status] = ...,
        activity: Optional[BaseActivity] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
        heartbeat_timeout: float = ...,
        guild_ready_timeout: float = ...,
        guild_subscriptions: bool = ...,
        assume_unsync_clock: bool = ...,
    ) -> None: ...
    @property
    def latency(self) -> float: ...
    @property
    def latencies(self) -> List[Tuple[int, float]]: ...
    def get_shard(self, shard_id: int) -> Optional[ShardInfo]: ...
    @property
    def shards(self) -> Mapping[int, ShardInfo]: ...
    async def request_offline_members(self, *guilds: Guild) -> None: ...
    async def launch_shard(
        self, gateway: str, shard_id: int, *, initial: bool = ...
    ) -> None: ...
    async def launch_shards(self) -> None: ...
    async def connect(self, *, reconnect: bool = ...) -> None: ...
    async def close(self) -> None: ...
    async def change_presence(
        self,
        *,
        activity: Optional[BaseActivity] = ...,
        status: Optional[Status] = ...,
        afk: bool = ...,
        shard_id: Optional[int] = ...,
    ) -> None: ...
    def is_ws_ratelimited(self) -> bool: ...



from datetime import datetime
from typing import Any, Optional, Type, TypeVar

from .asset import _VALID_ANIMATED_ICON_FORMATS, _VALID_STATIC_ICON_FORMATS, Asset
from .http import _PartialEmojiDict

class _EmojiTag: ...

_PE = TypeVar('_PE', bound=PartialEmoji)

class PartialEmoji(_EmojiTag):
    animated: bool
    name: Optional[str]
    id: Optional[int]
    @classmethod
    def from_dict(cls: Type[_PE], data: _PartialEmojiDict) -> _PE: ...
    def to_dict(self) -> _PartialEmojiDict: ...
    @classmethod
    def with_state(
        cls: Type[_PE],
        state: Any,
        *,
        name: Optional[str],
        animated: bool = ...,
        id: Optional[int] = ...,
    ) -> _PE: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...
    def is_custom_emoji(self) -> bool: ...
    def is_unicode_emoji(self) -> bool: ...
    @property
    def created_at(self) -> Optional[datetime]: ...
    @property
    def url(self) -> Asset: ...
    def url_as(
        self,
        *,
        format: Optional[_VALID_ANIMATED_ICON_FORMATS] = ...,
        static_format: _VALID_STATIC_ICON_FORMATS = ...,
    ) -> Asset: ...



import io
import threading
from typing import (
    Any,
    BinaryIO,
    Callable,
    ClassVar,
    Optional,
    Tuple,
    Type,
    TypeVar,
    Union,
    overload,
)
from typing_extensions import Literal

from .voice_client import VoiceClient

_FOA = TypeVar('_FOA', bound=FFmpegOpusAudio)
_MethodCallback = Callable[
    [Union[str, io.BufferedIOBase], str], Tuple[Optional[str], Optional[int]]
]
_OpusMethods = Literal['native', 'fallback']

class AudioSource:
    def read(self) -> bytes: ...
    def is_opus(self) -> bool: ...
    def cleanup(self) -> None: ...
    def __del__(self) -> None: ...

class PCMAudio(AudioSource):
    stream: BinaryIO
    def __init__(self, stream: BinaryIO) -> None: ...
    def read(self) -> bytes: ...

class FFmpegAudio(AudioSource):
    def __init__(
        self, source: Any, *, executable: str = ..., args: Any, **subprocess_kwargs: Any
    ) -> None: ...
    def cleanup(self) -> None: ...

class FFmpegPCMAudio(FFmpegAudio):
    @overload
    def __init__(
        self,
        source: io.BufferedIOBase,
        *,
        executable: str = ...,
        pipe: bool,
        stderr: Optional[BinaryIO] = ...,
        before_options: Optional[str] = ...,
        options: Optional[str] = ...,
    ) -> None: ...
    @overload
    def __init__(
        self,
        source: str,
        *,
        executable: str = ...,
        stderr: Optional[BinaryIO] = ...,
        before_options: Optional[str] = ...,
        options: Optional[str] = ...,
    ) -> None: ...
    def read(self) -> bytes: ...
    def is_opus(self) -> bool: ...

class FFmpegOpusAudio(FFmpegAudio):
    @overload
    def __init__(
        self,
        source: io.BufferedIOBase,
        *,
        bitrate: int = ...,
        codec: Optional[str] = ...,
        executable: str = ...,
        pipe: bool = ...,
        stderr: Optional[BinaryIO] = ...,
        before_options: Optional[str] = ...,
        options: Optional[str] = ...,
    ) -> None: ...
    @overload
    def __init__(
        self,
        source: str,
        *,
        bitrate: int = ...,
        codec: Optional[str] = ...,
        executable: str = ...,
        stderr: Optional[BinaryIO] = ...,
        before_options: Optional[str] = ...,
        options: Optional[str] = ...,
    ) -> None: ...
    @overload
    @classmethod
    async def from_probe(
        cls: Type[_FOA],
        source: io.BufferedIOBase,
        *,
        method: Optional[Union[_OpusMethods, _MethodCallback]] = ...,
        executable: str = ...,
        pipe: bool = ...,
        stderr: Optional[BinaryIO] = ...,
        before_options: Optional[str] = ...,
        options: Optional[str] = ...,
    ) -> _FOA: ...
    @overload
    @classmethod
    async def from_probe(
        cls: Type[_FOA],
        source: str,
        *,
        method: Optional[Union[_OpusMethods, _MethodCallback]] = ...,
        executable: str = ...,
        pipe: bool = ...,
        stderr: Optional[BinaryIO] = ...,
        before_options: Optional[str] = ...,
        options: Optional[str] = ...,
    ) -> _FOA: ...
    @overload
    @classmethod
    async def probe(
        cls,
        source: io.BufferedIOBase,
        *,
        method: Optional[Union[_OpusMethods, _MethodCallback]] = ...,
        executable: Optional[str] = ...,
    ) -> Tuple[Optional[str], Optional[int]]: ...
    @overload
    @classmethod
    async def probe(
        cls,
        source: str,
        *,
        method: Optional[Union[_OpusMethods, _MethodCallback]] = ...,
        executable: Optional[str] = ...,
    ) -> Tuple[Optional[str], Optional[int]]: ...
    def read(self) -> bytes: ...
    def is_opus(self) -> bool: ...

class PCMVolumeTransformer(AudioSource):
    original: AudioSource
    volume: float
    def __init__(self, original: AudioSource, volume: float = ...) -> None: ...
    def cleanup(self) -> None: ...
    def read(self) -> bytes: ...

class AudioPlayer(threading.Thread):
    DELAY: ClassVar[float]

    daemon: bool
    source: AudioSource
    client: VoiceClient
    loops: int
    def run(self) -> None: ...
    def stop(self) -> None: ...
    def pause(self, *, update_speaking: bool = ...) -> None: ...
    def resume(self, *, update_speaking: bool = ...) -> None: ...
    def is_playing(self) -> bool: ...
    def is_paused(self) -> bool: ...



import datetime
from typing import Any, ClassVar, Dict, List, Type, TypeVar, Union, type_check_only
from typing_extensions import Final, Protocol

from .colour import Colour
from .http import _EmbedDict

class _EmptyEmbed:
    def __bool__(self) -> bool: ...
    def __len__(self) -> int: ...

EmptyEmbed: Final[_EmptyEmbed]

class EmbedProxy:
    def __init__(self, layer: Dict[str, Any]) -> None: ...
    def __len__(self) -> int: ...
    def __getattr__(self, attr: str) -> _EmptyEmbed: ...

_E = TypeVar('_E', bound=Embed)

# TODO: remove this comment when a new version of black comes out
@type_check_only
class _EmbedFooterData(Protocol):
    text: Union[str, _EmptyEmbed]
    icon_url: Union[str, _EmptyEmbed]

@type_check_only
class _EmbedImageData(Protocol):
    url: Union[str, _EmptyEmbed]
    proxy_url: Union[str, _EmptyEmbed]
    height: Union[int, _EmptyEmbed]
    width: Union[int, _EmptyEmbed]

@type_check_only
class _EmbedVideoData(Protocol):
    url: Union[str, _EmptyEmbed]
    height: Union[int, _EmptyEmbed]
    width: Union[int, _EmptyEmbed]

@type_check_only
class _EmbedProviderData(Protocol):
    name: Union[str, _EmptyEmbed]
    url: Union[str, _EmptyEmbed]

@type_check_only
class _EmbedAuthorData(Protocol):
    name: Union[str, _EmptyEmbed]
    url: Union[str, _EmptyEmbed]
    icon_url: Union[str, _EmptyEmbed]
    proxy_icon_url: Union[str, _EmptyEmbed]

@type_check_only
class _EmbedFieldData(Protocol):
    name: Union[str, _EmptyEmbed]
    value: Union[str, _EmptyEmbed]
    inline: Union[bool, _EmptyEmbed]

class Embed:
    title: Union[str, _EmptyEmbed]
    type: str
    description: Union[str, _EmptyEmbed]
    url: Union[str, _EmptyEmbed]
    colour: Union[int, Colour, _EmptyEmbed]
    color: Union[int, Colour, _EmptyEmbed]
    timestamp: Union[datetime.datetime, _EmptyEmbed]

    Empty: ClassVar[_EmptyEmbed]
    def __init__(
        self,
        *,
        color: Union[int, Colour, _EmptyEmbed] = ...,
        colour: Union[int, Colour, _EmptyEmbed] = ...,
        title: Union[object, _EmptyEmbed] = ...,
        type: str = ...,
        url: Union[object, _EmptyEmbed] = ...,
        description: Union[object, _EmptyEmbed] = ...,
        timestamp: Union[datetime.datetime, _EmptyEmbed] = ...,
    ) -> None: ...
    @classmethod
    def from_dict(cls: Type[_E], data: _EmbedDict) -> _E: ...
    def copy(self) -> Embed: ...
    def __len__(self) -> int: ...
    @property
    def footer(self) -> _EmbedFooterData: ...
    def set_footer(
        self: _E,
        *,
        text: Union[str, _EmptyEmbed] = ...,
        icon_url: Union[str, _EmptyEmbed] = ...,
    ) -> _E: ...
    @property
    def image(self) -> _EmbedImageData: ...
    def set_image(self: _E, *, url: Union[str, _EmptyEmbed]) -> _E: ...
    @property
    def thumbnail(self) -> _EmbedImageData: ...
    def set_thumbnail(self: _E, *, url: Union[str, _EmptyEmbed]) -> _E: ...
    @property
    def video(self) -> _EmbedVideoData: ...
    @property
    def provider(self) -> _EmbedProviderData: ...
    @property
    def author(self) -> _EmbedAuthorData: ...
    def set_author(
        self: _E,
        *,
        name: str,
        url: Union[str, _EmptyEmbed] = ...,
        icon_url: Union[str, _EmptyEmbed] = ...,
    ) -> _E: ...
    def remove_author(self: _E) -> _E: ...
    @property
    def fields(self) -> List[_EmbedFieldData]: ...
    def add_field(self: _E, *, name: str, value: str, inline: bool = ...) -> _E: ...
    def insert_field_at(
        self: _E, index: int, *, name: str, value: str, inline: bool = ...
    ) -> _E: ...
    def clear_fields(self) -> None: ...
    def remove_field(self, index: int) -> None: ...
    def set_field_at(
        self: _E, index: int, *, name: str, value: str, inline: bool = ...
    ) -> _E: ...
    def to_dict(self) -> _EmbedDict: ...



from typing import Any, Optional, Union, type_check_only
from typing_extensions import Protocol, TypedDict

from .abc import Snowflake
from .emoji import Emoji
from .iterators import ReactionIterator
from .message import Message
from .partial_emoji import PartialEmoji

# TODO: remove this comment when a new version of black comes out
@type_check_only
class _RequiredReactionData(TypedDict):
    me: bool

@type_check_only
class _ReactionData(_RequiredReactionData, total=False):
    count: int

@type_check_only
class _UserProtocol(Protocol):
    id: int

class Reaction:
    emoji: Union[Emoji, PartialEmoji, str]
    count: int
    me: bool
    message: Message
    @property
    def custom_emoji(self) -> bool: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...
    async def remove(self, user: _UserProtocol) -> None: ...
    async def clear(self) -> None: ...
    def users(
        self, limit: Optional[int] = ..., after: Optional[Snowflake] = ...
    ) -> ReactionIterator: ...



import asyncio
from typing import (
    Any,
    BinaryIO,
    ClassVar,
    Coroutine,
    Dict,
    Iterable,
    List,
    Optional,
    Tuple,
    Union,
    type_check_only,
)
from typing_extensions import TypedDict

import aiohttp

from .file import File
from .mentions import _AllowedMentionsDict
from .message import _MessageReferenceDict

# TODO: remove this comment when a new version of black comes out
@type_check_only
class _ApplicationDict(TypedDict):
    id: int
    cover_image: str
    description: str
    icon: str
    name: str

@type_check_only
class _BaseUserDict(TypedDict):
    id: int
    username: str
    discriminator: str
    avatar: Optional[str]

@type_check_only
class _UserDict(_BaseUserDict, total=False):
    bot: bool
    mfa_enabled: bool
    verified: bool
    email: str
    premium: bool

@type_check_only
class _BaseApplicationInfoDict(TypedDict):
    id: int
    name: str
    bot_public: bool
    bot_require_code_grant: bool
    owner: _UserDict

@type_check_only
class _ApplicationInfoDict(_BaseApplicationInfoDict, total=False):
    icon: str
    description: str
    rpc_origins: List[str]

@type_check_only
class _AttachmentDict(TypedDict):
    id: int
    filename: str
    size: int
    url: str
    proxy_url: str
    height: Optional[int]
    width: Optional[int]

@type_check_only
class _EmbedFooterDict(TypedDict, total=False):
    text: str
    icon_url: str
    proxy_icon_url: str

@type_check_only
class _BaseEmbedMediaDict(TypedDict):
    url: str

@type_check_only
class _EmbedMediaDict(_BaseEmbedMediaDict, total=False):
    height: int
    width: int

@type_check_only
class _EmbedImageDict(_EmbedMediaDict, total=False):
    proxy_url: str

@type_check_only
class _EmbedProviderDict(TypedDict):
    name: str
    url: str

@type_check_only
class _EmbedAuthorDict(TypedDict, total=False):
    name: str
    url: str
    icon_url: str
    proxy_icon_url: str

@type_check_only
class _BaseEmbedFieldDict(TypedDict):
    name: str
    value: str

@type_check_only
class _EmbedFieldDict(_BaseEmbedFieldDict, total=False):
    inline: bool

@type_check_only
class _EmbedDict(TypedDict, total=False):
    type: str
    title: str
    description: str
    url: str
    timestamp: str
    color: int
    footer: _EmbedFooterDict
    image: _EmbedImageDict
    thumbnail: _EmbedImageDict
    video: _EmbedMediaDict
    provider: _EmbedProviderDict
    author: _EmbedAuthorDict
    fields: List[_EmbedFieldDict]

@type_check_only
class _PositionDict(TypedDict):
    id: int
    position: int

@type_check_only
class _OverwriteDict(TypedDict):
    id: int
    type: str
    allow: int
    deny: int

@type_check_only
class _BaseChannelDict(TypedDict):
    id: int
    type: int

@type_check_only
class _ChannelDict(_BaseChannelDict, total=False):
    guild_id: int
    position: int
    permission_overwrites: List[_OverwriteDict]
    name: str
    topic: str
    nsfw: bool
    last_message_id: Optional[int]
    bitrate: int
    user_limit: int
    recipients: List[_UserDict]
    icon: Optional[str]
    owner_id: int
    application_id: int
    parent_id: Optional[int]
    last_pin_timestamp: str

@type_check_only
class _ClientUserDict(TypedDict):
    afk_timeout: int
    animate_emojis: bool
    convert_emoticons: bool
    default_guilds_restricted: bool
    detect_platform_accounts: bool
    developer_mode: bool
    disable_games_tab: bool
    enable_tts_command: bool
    explicit_content_filter: int
    friend_source_flags: int
    gif_auto_play: bool
    guild_positions: List[int]
    inline_attachment_media: bool
    inline_embed_media: bool
    locale: str
    message_display_compact: bool
    render_embeds: bool
    render_reactions: bool
    restricted_guilds: List[int]
    show_current_game: bool
    status: str
    theme: str
    timezone_offset: int

@type_check_only
class _BaseMessageDict(TypedDict):
    id: int
    channel_id: int
    author: _UserDict
    content: str
    timestamp: str
    edited_timestamp: Optional[str]
    tts: bool
    mention_everyone: bool
    mentions: List[_UserDict]
    mention_roles: List[int]
    attachments: List[_AttachmentDict]
    embeds: List[_EmbedDict]
    pinned: bool
    type: int

@type_check_only
class _BaseMessageActivityDict(TypedDict):
    type: int

@type_check_only
class _MessageActivityDict(_BaseMessageActivityDict, total=False):
    party_id: str

@type_check_only
class _MessageCallDict(TypedDict, total=False):
    participants: List[int]

@type_check_only
class _BasePartialEmojiDict(TypedDict):
    id: Optional[int]
    name: str

@type_check_only
class _PartialEmojiDict(_BasePartialEmojiDict, total=False):
    animated: bool

@type_check_only
class _ReactionDict(TypedDict):
    count: int
    me: bool
    emoji: _PartialEmojiDict

@type_check_only
class _BaseEmojiDict(TypedDict):
    id: int
    name: str

@type_check_only
class _EmojiDict(_BaseEmojiDict, total=False):
    animated: bool
    roles: List[int]
    user: _UserDict
    require_colons: bool
    managed: bool

@type_check_only
class _MessageDict(_BaseMessageDict, total=False):
    reactions: List[_ReactionDict]
    nonce: int
    webhook_id: int
    activity: _MessageActivityDict
    application: Any
    call: _MessageCallDict
    guild_id: int

@type_check_only
class _BaseGuildMemberDict(TypedDict):
    user: _UserDict
    roles: List[int]
    joined_at: str
    deaf: bool
    mute: bool

@type_check_only
class _GuildMemberDict(_BaseGuildMemberDict, total=False):
    nick: str
    guild_id: str

@type_check_only
class _RoleDict(TypedDict):
    id: int
    name: str
    color: int
    hoist: bool
    position: int
    permissions: int
    managed: bool
    mentionable: bool

@type_check_only
class _BaseInviteDict(TypedDict):
    code: str
    guild: Any
    channel: Any

@type_check_only
class _InviteDict(_BaseInviteDict, total=False):
    approximate_presence_count: int
    approximate_member_count: int

@type_check_only
class _InviteMetaDict(_InviteDict):
    inviter: _UserDict
    uses: int
    max_uses: int
    max_age: int
    temporary: bool
    created_at: str
    revoked: bool

@type_check_only
class _BaseWebhookDict(TypedDict):
    id: int
    token: str

@type_check_only
class _WebhookDict(_BaseWebhookDict, total=False):
    channel_id: int
    name: Optional[str]
    avatar: Optional[str]
    guild_id: int
    user: _UserDict

@type_check_only
class _GuildBanDict(TypedDict):
    reason: Optional[str]
    user: _UserDict

@type_check_only
class _GuildPruneDict(TypedDict):
    pruned: int

@type_check_only
class _TimestampsDict(TypedDict, total=False):
    start: int
    end: int

@type_check_only
class _ActivityPartyDict(TypedDict, total=False):
    id: str
    size: List[int]

@type_check_only
class _ActivityAssetsDict(TypedDict, total=False):
    large_image: str
    large_text: str
    small_image: str
    small_text: str

@type_check_only
class _BaseActivityDict(TypedDict):
    name: str
    type: int

@type_check_only
class _ActivityDict(_BaseActivityDict, total=False):
    url: Optional[str]
    timestamps: _TimestampsDict
    application_id: int
    details: Optional[str]
    state: Optional[str]
    party: _ActivityPartyDict
    assets: _ActivityAssetsDict

@type_check_only
class _SpotifyActivityDict(TypedDict, total=False):
    name: str
    url: Optional[str]
    timestamps: _TimestampsDict
    application_id: int
    details: Optional[str]
    state: Optional[str]
    party: _ActivityPartyDict
    assets: _ActivityAssetsDict
    flags: int
    sync_id: str
    session_id: str

@type_check_only
class _BaseCustomActivityDict(TypedDict):
    type: int
    name: Optional[str]

@type_check_only
class _CustomActivityDict(_BaseCustomActivityDict, total=False):
    state: str
    emoji: _EmojiDict

@type_check_only
class _PresenceUpdateDict(TypedDict):
    user: _UserDict
    roles: List[int]
    game: Optional[_ActivityDict]
    guild_id: int
    status: str

@type_check_only
class _BaseGuildDict(TypedDict):
    id: int
    name: str
    icon: Optional[str]
    splash: Optional[str]
    owner_id: int
    region: str
    afk_channel_id: Optional[int]
    afk_timeout: int
    verification_level: int
    default_message_notifications: int
    explicit_content_filter: int
    roles: List[_RoleDict]
    emojis: List[_EmojiDict]
    features: List[str]
    mfa_level: int
    application_id: Optional[int]
    system_channel_id: Optional[int]

@type_check_only
class _GuildDict(_BaseGuildDict, total=False):
    owner: bool
    permissions: int
    embed_enabled: bool
    embed_channel_id: int
    widget_enabled: bool
    widget_channel_id: int
    joined_at: str
    large: bool
    unavailable: bool
    member_count: int
    voice_states: List[Any]
    members: List[_GuildMemberDict]
    channels: List[_ChannelDict]
    presences: List[_PresenceUpdateDict]

@type_check_only
class _BaseAuditLogChangeDict(TypedDict):
    key: str

@type_check_only
class _AuditLogChangeDict(_BaseAuditLogChangeDict, total=False):
    new_value: Any
    old_value: Any

@type_check_only
class _BaseAuditLogEntryDict(TypedDict):
    target_id: Optional[str]
    user_id: int
    id: int
    action_type: int

@type_check_only
class _AuditLogEntryDict(_BaseAuditLogEntryDict, total=False):
    changes: List[_AuditLogChangeDict]
    options: Any
    reason: str

@type_check_only
class _AuditLogDict(TypedDict):
    webhooks: List[_WebhookDict]
    users: List[_UserDict]
    audit_log_entries: List[_AuditLogEntryDict]

@type_check_only
class _CurrentUserGuildDict(TypedDict):
    id: int
    name: str
    icon: str
    owner: bool
    permissions: int

@type_check_only
class _BaseVoiceStateDict(TypedDict):
    request_to_speak_timestamp: Optional[str]
    channel_id: int

@type_check_only
class _VoiceStateDict(_BaseVoiceStateDict, total=False):
    suppress: bool

@type_check_only
class _WidgetChannelDict(TypedDict):
    id: int
    name: str
    position: int

@type_check_only
class _BaseWidgetMemberDict(TypedDict):
    id: int
    username: str
    discriminator: str
    nick: str
    status: str

@type_check_only
class _WidgetMemberDict(_BaseWidgetMemberDict, total=False):
    channel_id: int
    avatar: str
    bot: bool
    deaf: bool
    self_deaf: bool
    mute: bool
    self_mute: bool
    suppress: bool
    activity: Union[_ActivityDict, _SpotifyActivityDict]

@type_check_only
class _BaseWidgetDict(TypedDict):
    id: int
    name: str

@type_check_only
class _WidgetDict(_BaseWidgetDict, total=False):
    instant_invite: Optional[str]
    channels: List[_WidgetChannelDict]
    members: List[_WidgetMemberDict]

@type_check_only
class _BaseTemplateDict(TypedDict):
    code: str
    usage_count: int
    name: str
    description: Optional[str]
    source_guild_id: int
    serialized_source_guild: _GuildDict

@type_check_only
class _TemplateDict(_BaseTemplateDict, total=False):
    creator: _UserDict
    created_at: str
    updated_at: str

@type_check_only
class _BaseCreateTemplateDict(TypedDict):
    name: str

@type_check_only
class _CreateTemplateDict(TypedDict, total=False):
    description: Optional[str]

@type_check_only
class _IntegrationAccountDict(TypedDict):
    id: int
    name: str

@type_check_only
class _BaseIntegrationDict(TypedDict):
    id: int
    name: str
    type: str
    enabled: bool
    syncing: bool
    role_id: bool
    expire_behavior: int
    expire_grace_period: int
    user: _UserDict
    account: _IntegrationAccountDict
    synced_at: str

@type_check_only
class _IntegrationDict(_BaseIntegrationDict, total=False):
    enable_emoticons: bool

async def json_or_text(response: Any) -> Any: ...

class Route:
    BASE: ClassVar[str]
    channel_id: Optional[int]
    guild_id: Optional[int]
    def __init__(self, method: str, path: str, **parameters: Any) -> None: ...
    @property
    def bucket(self) -> str: ...

class MaybeUnlock:
    def __init__(self, lock: asyncio.Lock) -> None: ...
    def __enter__(self) -> MaybeUnlock: ...
    def defer(self) -> None: ...
    def __exit__(self, type: Any, value: Any, traceback: Any) -> None: ...

class HTTPClient:
    SUCCESS_LOG: ClassVar[str]
    REQUEST_LOG: ClassVar[str]

    loop: asyncio.AbstractEventLoop
    connector: Optional[aiohttp.BaseConnector]
    token: Optional[str]
    bot_token: bool
    proxy: Optional[str]
    proxy_auth: Optional[aiohttp.BasicAuth]
    user_agent: str
    use_clock: bool
    def recreate(self) -> None: ...
    async def ws_connect(
        self, url: str, *, compress: int = ...
    ) -> aiohttp.ClientWebSocketResponse: ...
    async def request(
        self,
        route: Route,
        *,
        files: Optional[Iterable[File]] = ...,
        form: Optional[List[Dict[str, Any]]] = ...,
        **kwargs: Any,
    ) -> Any: ...
    async def get_from_cdn(self, url: str) -> bytes: ...
    async def close(self) -> None: ...
    async def static_login(self, token: str, *, bot: bool) -> Any: ...
    def logout(self) -> Coroutine[Any, Any, Any]: ...
    def start_group(
        self, user_id: int, recipients: List[Any]
    ) -> Coroutine[Any, Any, _ChannelDict]: ...
    def leave_group(self, channel_id: int) -> Coroutine[Any, Any, _ChannelDict]: ...
    def add_group_recipient(
        self, channel_id: int, user_id: int
    ) -> Coroutine[Any, Any, Any]: ...
    def remove_group_recipient(
        self, channel_id: int, user_id: int
    ) -> Coroutine[Any, Any, None]: ...
    def edit_group(
        self, channel_id: int, **options: Any
    ) -> Coroutine[Any, Any, _ChannelDict]: ...
    def convert_group(self, channel_id: int) -> Coroutine[Any, Any, Any]: ...
    def start_private_message(
        self, user_id: int
    ) -> Coroutine[Any, Any, _ChannelDict]: ...
    def send_message(
        self,
        channel_id: int,
        content: str,
        *,
        tts: bool = ...,
        embed: Optional[Dict[str, Any]] = ...,
        nonce: Optional[int] = ...,
        allowed_mentions: Optional[_AllowedMentionsDict] = ...,
        message_reference: Optional[_MessageReferenceDict] = ...,
    ) -> Coroutine[Any, Any, _MessageDict]: ...
    def send_typing(self, channel_id: int) -> Coroutine[Any, Any, None]: ...
    def send_files(
        self,
        channel_id: int,
        *,
        files: List[Tuple[BinaryIO, str]],
        content: Optional[str] = ...,
        tts: bool = ...,
        embed: Optional[Dict[str, Any]] = ...,
        nonce: Optional[int] = ...,
        allowed_mentions: Optional[_AllowedMentionsDict] = ...,
        message_reference: Optional[_MessageReferenceDict] = ...,
    ) -> Coroutine[Any, Any, _MessageDict]: ...
    async def ack_message(self, channel_id: int, message_id: int) -> None: ...
    def ack_guild(self, guild_id: int) -> Coroutine[Any, Any, Any]: ...
    def delete_message(
        self, channel_id: int, message_id: int, *, reason: Optional[str] = ...
    ) -> Coroutine[Any, Any, None]: ...
    def delete_messages(
        self, channel_id: int, message_ids: List[int], *, reason: Optional[str] = ...
    ) -> Coroutine[Any, Any, None]: ...
    def edit_message(
        self, channel_id: int, message_id: int, **fields: Any
    ) -> Coroutine[Any, Any, _MessageDict]: ...
    def add_reaction(
        self, channel_id: int, message_id: int, emoji: str
    ) -> Coroutine[Any, Any, None]: ...
    def remove_reaction(
        self, channel_id: int, message_id: int, emoji: str, member_id: int
    ) -> Coroutine[Any, Any, None]: ...
    def remove_own_reaction(
        self, channel_id: int, message_id: int, emoji: str
    ) -> Coroutine[Any, Any, None]: ...
    def get_reaction_users(
        self,
        channel_id: int,
        message_id: int,
        emoji: str,
        limit: int,
        after: Optional[int] = ...,
    ) -> Coroutine[Any, Any, _UserDict]: ...
    def clear_reactions(
        self, channel_id: int, message_id: int
    ) -> Coroutine[Any, Any, None]: ...
    def clear_single_reaction(
        self, channel_id: int, message_id: int, emoji: str
    ) -> Coroutine[Any, Any, None]: ...
    def get_message(
        self, channel_id: int, message_id: int
    ) -> Coroutine[Any, Any, _MessageDict]: ...
    def get_channel(self, channel_id: int) -> Coroutine[Any, Any, _ChannelDict]: ...
    def logs_from(
        self,
        channel_id: int,
        limit: int,
        before: Optional[int] = ...,
        after: Optional[int] = ...,
        around: Optional[int] = ...,
    ) -> Coroutine[Any, Any, List[_MessageDict]]: ...
    def publish_message(
        self, channel_id: int, message_id: int
    ) -> Coroutine[Any, Any, None]: ...
    def pin_message(
        self, channel_id: int, message_id: int, reason: Optional[str] = ...
    ) -> Coroutine[Any, Any, None]: ...
    def unpin_message(
        self, channel_id: int, message_id: int, reason: Optional[str] = ...
    ) -> Coroutine[Any, Any, None]: ...
    def pins_from(self, channel_id: int) -> Coroutine[Any, Any, List[_MessageDict]]: ...
    def kick(
        self, user_id: int, guild_id: int, reason: Optional[str] = ...
    ) -> Coroutine[Any, Any, None]: ...
    def ban(
        self,
        user_id: int,
        guild_id: int,
        delete_message_days: int = ...,
        reason: Optional[str] = ...,
    ) -> Coroutine[Any, Any, None]: ...
    def unban(
        self, user_id: int, guild_id: int, *, reason: Optional[str] = ...
    ) -> Coroutine[Any, Any, None]: ...
    def guild_voice_state(
        self,
        user_id: int,
        guild_id: int,
        *,
        mute: Optional[bool] = ...,
        deafen: Optional[bool] = ...,
        reason: Optional[str] = ...,
    ) -> Coroutine[Any, Any, None]: ...
    def edit_profile(
        self, password: str, username: str, avatar: str, **fields: Any
    ) -> Coroutine[Any, Any, _UserDict]: ...
    def change_my_nickname(
        self, guild_id: int, nickname: str, *, reason: Optional[str] = ...
    ) -> Coroutine[Any, Any, str]: ...
    def change_nickname(
        self, guild_id: int, user_id: int, nickname: str, *, reason: Optional[str] = ...
    ) -> Coroutine[Any, Any, None]: ...
    def edit_member(
        self, guild_id: int, user_id: int, *, reason: Optional[str] = ..., **fields: Any
    ) -> Coroutine[Any, Any, None]: ...
    def edit_my_voice_state(
        self, guild_id: int, payload: _VoiceStateDict
    ) -> Coroutine[Any, Any, None]: ...
    def edit_voice_state(
        self, guild_id: int, user_id: int, payload: _VoiceStateDict
    ) -> Coroutine[Any, Any, None]: ...
    def edit_channel(
        self, channel_id: int, *, reason: Optional[str] = ..., **options: Any
    ) -> Coroutine[Any, Any, _ChannelDict]: ...
    def bulk_channel_update(
        self, guild_id: int, data: List[_PositionDict], *, reason: Optional[str] = ...
    ) -> Coroutine[Any, Any, None]: ...
    def create_channel(
        self,
        guild_id: int,
        channel_type: int,
        *,
        reason: Optional[str] = ...,
        name: str,
        parent_id: Optional[int] = ...,
        topic: Optional[str] = ...,
        bitrate: Optional[int] = ...,
        nsfw: Optional[bool] = ...,
        user_limit: Optional[int] = ...,
        position: Optional[int] = ...,
        permission_overwrites: Optional[List[_OverwriteDict]] = ...,
        rate_limit_per_user: Optional[int] = ...,
        rtc_region: Optional[int] = ...,
    ) -> Coroutine[Any, Any, _ChannelDict]: ...
    def delete_channel(
        self, channel_id: int, *, reason: Optional[str] = ...
    ) -> Coroutine[Any, Any, _ChannelDict]: ...
    def create_webhook(
        self,
        channel_id: int,
        *,
        name: str,
        avatar: Optional[str] = ...,
        reason: Optional[str] = ...,
    ) -> Coroutine[Any, Any, _WebhookDict]: ...
    def channel_webhooks(
        self, channel_id: int
    ) -> Coroutine[Any, Any, List[_WebhookDict]]: ...
    def guild_webhooks(
        self, guild_id: int
    ) -> Coroutine[Any, Any, List[_WebhookDict]]: ...
    def get_webhook(self, webhook_id: int) -> Coroutine[Any, Any, _WebhookDict]: ...
    def follow_webhook(
        self, channel_id: int, webhook_channel_id: int, reason: Optional[str] = ...
    ) -> Coroutine[Any, Any, _WebhookDict]: ...
    def get_guilds(
        self, limit: int, before: Optional[int] = ..., after: Optional[int] = ...
    ) -> Coroutine[Any, Any, List[_CurrentUserGuildDict]]: ...
    def leave_guild(self, guild_id: int) -> Coroutine[Any, Any, None]: ...
    def get_guild(self, guild_id: int) -> Coroutine[Any, Any, _GuildDict]: ...
    def delete_guild(self, guild_id: int) -> Coroutine[Any, Any, None]: ...
    def create_guild(
        self, name: str, region: str, icon: str
    ) -> Coroutine[Any, Any, _GuildDict]: ...
    def edit_guild(
        self, guild_id: int, *, reason: Optional[str] = ..., **fields: Any
    ) -> Coroutine[Any, Any, _GuildDict]: ...
    def get_template(self, code: str) -> Coroutine[Any, Any, _TemplateDict]: ...
    def guild_templates(
        self, guild_id: int
    ) -> Coroutine[Any, Any, List[_TemplateDict]]: ...
    def create_template(
        self, guild_id: int, payload: _CreateTemplateDict
    ) -> Coroutine[Any, Any, _TemplateDict]: ...
    def sync_template(
        self, guild_id: int, code: str
    ) -> Coroutine[Any, Any, _TemplateDict]: ...
    def edit_template(
        self, guild_id: int, code: str, payload: _CreateTemplateDict
    ) -> Coroutine[Any, Any, _TemplateDict]: ...
    def delete_template(
        self, guild_id: int, code: str
    ) -> Coroutine[Any, Any, _TemplateDict]: ...
    def create_from_template(
        self, code: str, name: str, region: str, icon: str
    ) -> Coroutine[Any, Any, _GuildDict]: ...
    def get_bans(self, guild_id: int) -> Coroutine[Any, Any, List[_GuildBanDict]]: ...
    def get_ban(
        self, user_id: int, guild_id: int
    ) -> Coroutine[Any, Any, _GuildBanDict]: ...
    def get_vanity_code(self, guild_id: int) -> Coroutine[Any, Any, _InviteDict]: ...
    def change_vanity_code(
        self, guild_id: int, code: str, *, reason: Optional[str] = ...
    ) -> Coroutine[Any, Any, Any]: ...
    def prune_members(
        self,
        guild_id: int,
        days: int,
        compute_prune_count: bool,
        roles: Optional[List[int]],
        *,
        reason: Optional[str] = ...,
    ) -> Coroutine[Any, Any, _GuildPruneDict]: ...
    def get_all_guild_channels(
        self, guild_id: int
    ) -> Coroutine[Any, Any, List[_ChannelDict]]: ...
    def get_members(
        self, guild_id: int, limit: int, after: Optional[int]
    ) -> Coroutine[Any, Any, List[_GuildMemberDict]]: ...
    def get_member(
        self, guild_id: int, member_id: int
    ) -> Coroutine[Any, Any, _GuildMemberDict]: ...
    def estimate_pruned_members(
        self, guild_id: int, days: int, roles: List[str]
    ) -> Coroutine[Any, Any, _GuildPruneDict]: ...
    def get_all_custom_emojis(
        self, guild_id: int
    ) -> Coroutine[Any, Any, _EmojiDict]: ...
    def get_custom_emoji(
        self, guild_id: int, emoji_id: int
    ) -> Coroutine[Any, Any, _EmojiDict]: ...
    def create_custom_emoji(
        self,
        guild_id: int,
        name: str,
        image: str,
        *,
        roles: Optional[List[int]] = ...,
        reason: Optional[str] = ...,
    ) -> Coroutine[Any, Any, _EmojiDict]: ...
    def delete_custom_emoji(
        self, guild_id: int, emoji_id: int, *, reason: Optional[str] = ...
    ) -> Coroutine[Any, Any, None]: ...
    def edit_custom_emoji(
        self,
        guild_id: int,
        emoji_id: int,
        *,
        name: str,
        roles: Optional[List[int]] = ...,
        reason: Optional[str] = ...,
    ) -> Coroutine[Any, Any, _EmojiDict]: ...
    def get_all_integrations(
        self, guild_id: int
    ) -> Coroutine[Any, Any, List[_IntegrationDict]]: ...
    def create_integration(
        self, guild_id: int, type: str, id: int
    ) -> Coroutine[Any, Any, None]: ...
    def edit_integration(
        self, guild_id: int, integration_id: int, **payload: Any
    ) -> Coroutine[Any, Any, None]: ...
    def sync_integration(
        self, guild_id: int, integration_id: int
    ) -> Coroutine[Any, Any, None]: ...
    def delete_integration(
        self, guild_id: int, integration_id: int
    ) -> Coroutine[Any, Any, None]: ...
    def get_audit_logs(
        self,
        guild_id: int,
        limit: int = ...,
        before: Optional[int] = ...,
        after: Optional[int] = ...,
        user_id: Optional[int] = ...,
        action_type: Optional[int] = ...,
    ) -> Coroutine[Any, Any, _AuditLogDict]: ...
    def get_widget(self, guild_id: int) -> Coroutine[Any, Any, _WidgetDict]: ...
    def create_invite(
        self,
        channel_id: int,
        *,
        reason: Optional[str] = ...,
        max_age: int = ...,
        max_uses: int = ...,
        temporary: bool = ...,
        unique: bool = ...,
    ) -> Coroutine[Any, Any, _InviteDict]: ...
    def get_invite(
        self, invite_id: str, *, with_counts: bool = ...
    ) -> Coroutine[Any, Any, _InviteDict]: ...
    def invites_from(
        self, guild_id: int
    ) -> Coroutine[Any, Any, List[_InviteMetaDict]]: ...
    def invites_from_channel(
        self, channel_id: int
    ) -> Coroutine[Any, Any, List[_InviteMetaDict]]: ...
    def delete_invite(
        self, invite_id: str, *, reason: Optional[str] = ...
    ) -> Coroutine[Any, Any, _InviteDict]: ...
    def get_roles(self, guild_id: int) -> Coroutine[Any, Any, List[_RoleDict]]: ...
    def edit_role(
        self, guild_id: int, role_id: int, *, reason: Optional[str] = ..., **fields: Any
    ) -> Coroutine[Any, Any, _RoleDict]: ...
    def delete_role(
        self, guild_id: int, role_id: int, *, reason: Optional[str] = ...
    ) -> Coroutine[Any, Any, None]: ...
    def replace_roles(
        self,
        user_id: int,
        guild_id: int,
        role_ids: List[int],
        *,
        reason: Optional[str] = ...,
    ) -> Coroutine[Any, Any, _GuildMemberDict]: ...
    def create_role(
        self, guild_id: int, *, reason: Optional[str] = ..., **fields: Any
    ) -> Coroutine[Any, Any, _RoleDict]: ...
    def move_role_position(
        self,
        guild_id: int,
        positions: List[_PositionDict],
        *,
        reason: Optional[str] = ...,
    ) -> Coroutine[Any, Any, List[_RoleDict]]: ...
    def add_role(
        self, guild_id: int, user_id: int, role_id: int, *, reason: Optional[str] = ...
    ) -> Coroutine[Any, Any, None]: ...
    def remove_role(
        self, guild_id: int, user_id: int, role_id: int, *, reason: Optional[str] = ...
    ) -> Coroutine[Any, Any, None]: ...
    def edit_channel_permissions(
        self,
        channel_id: int,
        target: int,
        allow: int,
        deny: int,
        type: str,
        *,
        reason: Optional[str] = ...,
    ) -> Coroutine[Any, Any, None]: ...
    def delete_channel_permissions(
        self, channel_id: int, target: int, *, reason: Optional[str] = ...
    ) -> Coroutine[Any, Any, None]: ...
    def move_member(
        self,
        user_id: int,
        guild_id: int,
        channel_id: int,
        *,
        reason: Optional[str] = ...,
    ) -> Coroutine[Any, Any, _GuildMemberDict]: ...
    def remove_relationship(self, user_id: int) -> Coroutine[Any, Any, Any]: ...
    def add_relationship(
        self, user_id: int, type: Optional[int] = ...
    ) -> Coroutine[Any, Any, Any]: ...
    def send_friend_request(
        self, username: str, discriminator: str
    ) -> Coroutine[Any, Any, Any]: ...
    def application_info(self) -> Coroutine[Any, Any, _ApplicationInfoDict]: ...
    async def get_gateway(
        self, *, encoding: str = ..., v: int = ..., zlib: bool = ...
    ) -> str: ...
    async def get_bot_gateway(
        self, *, encoding: str = ..., v: int = ..., zlib: bool = ...
    ) -> Tuple[int, str]: ...
    def get_user(self, user_id: int) -> Coroutine[Any, Any, _UserDict]: ...
    def get_user_profile(self, user_id: int) -> Coroutine[Any, Any, Any]: ...
    def get_mutual_friends(self, user_id: int) -> Coroutine[Any, Any, _UserDict]: ...
    def change_hypesquad_house(self, house_id: int) -> Coroutine[Any, Any, Any]: ...
    def leave_hypesquad_house(self) -> Coroutine[Any, Any, None]: ...
    def edit_settings(
        self,
        *,
        afk_timeout: int = ...,
        animate_emojis: bool = ...,
        convert_emoticons: bool = ...,
        default_guilds_restricted: bool = ...,
        detect_platform_accounts: bool = ...,
        developer_mode: bool = ...,
        disable_games_tab: bool = ...,
        enable_tts_command: bool = ...,
        explicit_content_filter: int = ...,
        friend_source_flags: int = ...,
        gif_auto_play: bool = ...,
        guild_positions: List[int] = ...,
        inline_attachment_media: bool = ...,
        inline_embed_media: bool = ...,
        locale: str = ...,
        message_display_compact: bool = ...,
        render_embeds: bool = ...,
        render_reactions: bool = ...,
        restricted_guilds: List[int] = ...,
        show_current_game: bool = ...,
        status: str = ...,
        theme: str = ...,
        timezone_offset: int = ...,
    ) -> Coroutine[Any, Any, _ClientUserDict]: ...



import datetime
from os import PathLike
from typing import BinaryIO, List, Optional, Type, TypeVar, Union, type_check_only
from typing_extensions import TypedDict

from .abc import User as _BaseUser
from .calls import CallMessage
from .channel import DMChannel, GroupChannel, TextChannel
from .embeds import Embed
from .emoji import Emoji
from .enums import MessageType
from .file import File
from .flags import MessageFlags
from .guild import Guild
from .member import Member
from .mentions import AllowedMentions
from .mixins import Hashable
from .partial_emoji import PartialEmoji
from .reaction import Reaction
from .role import Role
from .sticker import Sticker
from .user import ClientUser, User
from .utils import cached_slot_property

_MR = TypeVar('_MR', bound=MessageReference)

class Attachment(Hashable):
    id: int
    size: int
    height: Optional[int]
    width: Optional[int]
    filename: str
    url: str
    proxy_url: str
    content_type: Optional[str]
    def is_spoiler(self) -> bool: ...
    async def save(
        self,
        fp: Union[BinaryIO, PathLike[str], str],
        *,
        seek_begin: bool = ...,
        use_cached: bool = ...,
    ) -> int: ...
    async def read(self, *, use_cached: bool = ...) -> bytes: ...
    async def to_file(self, *, use_cached: bool = ..., spoiler: bool = ...) -> File: ...

class DeletedReferencedMessage:
    @property
    def id(self) -> Optional[int]: ...
    @property
    def channel_id(self) -> int: ...
    @property
    def guild_id(self) -> Optional[int]: ...

@type_check_only
class _BaseMessageReferenceDict(TypedDict):
    channel_id: int

@type_check_only
class _MessageReferenceDict(_BaseMessageReferenceDict, total=False):
    message_id: int
    guild_id: int
    fail_if_not_exists: bool

class MessageReference:
    message_id: Optional[int]
    channel_id: int
    guild_id: Optional[int]
    fail_if_not_exists: bool
    resolved: Optional[Union[Message, DeletedReferencedMessage]]
    def __init__(
        self,
        *,
        message_id: int,
        channel_id: int,
        guild_id: Optional[int] = ...,
        fail_if_not_exists: bool = ...,
    ) -> None: ...
    @classmethod
    def from_message(
        cls: Type[_MR], message: Message, *, fail_if_not_exists: bool = ...
    ) -> _MR: ...
    @property
    def cached_message(self) -> Optional[Message]: ...
    @property
    def jump_url(self) -> str: ...
    def to_dict(self) -> _MessageReferenceDict: ...
    to_message_reference_dict = to_dict

@type_check_only
class _MessageActivity(TypedDict, total=False):
    type: int
    party_id: str

@type_check_only
class _MessageApplication(TypedDict):
    id: str
    name: str
    description: str
    icon: str
    cover_image: str

@type_check_only
class _MessageShared:
    @property
    def jump_url(self) -> str: ...
    async def delete(self, *, delay: Optional[float] = ...) -> None: ...
    async def publish(self) -> None: ...
    async def pin(self, *, reason: Optional[str] = ...) -> None: ...
    async def unpin(self, *, reason: Optional[str] = ...) -> None: ...
    async def add_reaction(
        self, emoji: Union[Emoji, Reaction, PartialEmoji, str]
    ) -> None: ...
    async def remove_reaction(
        self, emoji: Union[Emoji, Reaction, PartialEmoji, str], member: _BaseUser
    ) -> None: ...
    async def clear_reaction(
        self, emoji: Union[Emoji, Reaction, PartialEmoji, str]
    ) -> None: ...
    async def clear_reactions(self) -> None: ...
    async def reply(
        self,
        content: Optional[object] = ...,
        *,
        tts: bool = ...,
        embed: Optional[Embed] = ...,
        file: Optional[File] = ...,
        files: Optional[List[File]] = ...,
        delete_after: Optional[float] = ...,
        nonce: Optional[int] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
        mention_author: Optional[bool] = ...,
    ) -> Message: ...
    def to_reference(self, *, fail_if_not_exists: bool = ...) -> MessageReference: ...
    def to_message_reference_dict(self) -> _MessageReferenceDict: ...

class Message(Hashable, _MessageShared):
    id: int
    tts: bool
    type: MessageType
    author: Union[User, Member]
    content: str
    nonce: Union[int, str]
    embeds: List[Embed]
    channel: Union[TextChannel, DMChannel, GroupChannel]
    call: Optional[CallMessage]
    reference: Optional[MessageReference]
    mention_everyone: bool
    mentions: List[Union[User, Member, ClientUser]]
    role_mentions: List[Role]
    webhook_id: Optional[int]
    attachments: List[Attachment]
    pinned: bool
    flags: MessageFlags
    reactions: List[Reaction]
    activity: Optional[_MessageActivity]
    application: Optional[_MessageApplication]
    stickers: List[Sticker]
    @cached_slot_property('_cs_guild')
    def guild(self) -> Optional[Guild]: ...
    @cached_slot_property('_cs_raw_mentions')
    def raw_mentions(self) -> List[int]: ...
    @cached_slot_property('_cs_raw_channel_mentions')
    def raw_channel_mentions(self) -> List[int]: ...
    @cached_slot_property('_cs_raw_role_mentions')
    def raw_role_mentions(self) -> List[int]: ...
    @cached_slot_property('_cs_channel_mentions')
    def channel_mentions(self) -> List[TextChannel]: ...
    @cached_slot_property('_cs_clean_content')
    def clean_content(self) -> str: ...
    @property
    def created_at(self) -> datetime.datetime: ...
    @property
    def edited_at(self) -> Optional[datetime.datetime]: ...
    @property
    def jump_url(self) -> str: ...
    def is_system(self) -> bool: ...
    @cached_slot_property('_cs_system_content')
    def system_content(self) -> str: ...
    async def edit(
        self,
        *,
        content: Optional[str] = ...,
        embed: Optional[Embed] = ...,
        suppress: bool = ...,
        delete_after: Optional[float] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> None: ...
    async def ack(self) -> None: ...

class PartialMessage(Hashable, _MessageShared):
    id: int
    channel: Union[TextChannel, DMChannel]
    @property
    def created_at(self) -> datetime.datetime: ...
    @cached_slot_property('_cs_guild')
    def guild(self) -> Optional[Guild]: ...
    async def fetch(self) -> Message: ...
    async def edit(
        self,
        *,
        content: Optional[str] = ...,
        embed: Optional[Embed] = ...,
        suppress: bool = ...,
        delete_after: Optional[float] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> Optional[Message]: ...



from .bot import AutoShardedBot as AutoShardedBot  # noqa: F401
from .bot import Bot as Bot  # noqa: F401
from .bot import when_mentioned as when_mentioned  # noqa: F401
from .bot import when_mentioned_or as when_mentioned_or  # noqa: F401
from .cog import *  # noqa: F401
from .context import Context as Context  # noqa: F401
from .converter import *  # noqa: F401
from .cooldowns import *  # noqa: F401
from .core import *  # noqa: F401
from .errors import *  # noqa: F401
from .help import *  # noqa: F401



import datetime
from typing import Any, List, Optional, Union, overload

from .colour import Colour
from .enums import ActivityType
from .http import (
    _ActivityAssetsDict,
    _ActivityDict,
    _ActivityPartyDict,
    _CustomActivityDict,
    _PartialEmojiDict,
    _SpotifyActivityDict,
    _TimestampsDict,
)
from .partial_emoji import PartialEmoji

class BaseActivity:
    @property
    def created_at(self) -> datetime.datetime: ...

class Activity(BaseActivity):
    application_id: int
    name: str
    url: str
    type: ActivityType
    state: str
    details: str
    timestamps: _TimestampsDict
    assets: _ActivityAssetsDict
    party: _ActivityPartyDict
    emoji: Optional[PartialEmoji]
    flags: int
    sync_id: Optional[str]
    session_id: Optional[str]
    def __init__(
        self,
        *,
        state: Optional[str] = ...,
        details: Optional[str] = ...,
        timestamps: _TimestampsDict = ...,
        assets: _ActivityAssetsDict = ...,
        party: _ActivityPartyDict = ...,
        application_id: Optional[Union[str, int]] = ...,
        name: Optional[str] = ...,
        url: Optional[str] = ...,
        flags: int = ...,
        sync_id: Optional[str] = ...,
        session_id: Optional[str] = ...,
        type: ActivityType = ...,
    ) -> None: ...
    def to_dict(self) -> _ActivityDict: ...
    @property
    def start(self) -> Optional[datetime.datetime]: ...
    @property
    def end(self) -> Optional[datetime.datetime]: ...
    @property
    def large_image_url(self) -> Optional[str]: ...
    @property
    def small_image_url(self) -> Optional[str]: ...
    @property
    def large_image_text(self) -> Optional[str]: ...
    @property
    def small_image_text(self) -> Optional[str]: ...

class Game(BaseActivity):
    name: str
    @overload
    def __init__(self, name: str, *, timestamps: _TimestampsDict) -> None: ...
    @overload
    def __init__(
        self,
        name: str,
        *,
        start: Optional[datetime.datetime] = ...,
        end: Optional[datetime.datetime] = ...,
    ) -> None: ...
    @property
    def type(self) -> ActivityType: ...
    @property
    def start(self) -> Optional[datetime.datetime]: ...
    @property
    def end(self) -> Optional[datetime.datetime]: ...
    def to_dict(self) -> _ActivityDict: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...

class Streaming(BaseActivity):
    platform: str
    name: str
    details: str
    game: Optional[str]
    url: str
    assets: _ActivityAssetsDict
    def __init__(
        self,
        *,
        name: str,
        url: str,
        details: Optional[str] = ...,
        assets: _ActivityAssetsDict = ...,
    ) -> None: ...
    @property
    def type(self) -> ActivityType: ...
    @property
    def twitch_name(self) -> Optional[str]: ...
    def to_dict(self) -> _ActivityDict: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...

class Spotify:
    @property
    def type(self) -> ActivityType: ...
    @property
    def created_at(self) -> datetime.datetime: ...
    @property
    def colour(self) -> Colour: ...
    @property
    def color(self) -> Colour: ...
    def to_dict(self) -> _SpotifyActivityDict: ...
    @property
    def name(self) -> str: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...
    @property
    def title(self) -> Optional[str]: ...
    @property
    def artists(self) -> List[str]: ...
    @property
    def artist(self) -> str: ...
    @property
    def album(self) -> str: ...
    @property
    def album_cover_url(self) -> str: ...
    @property
    def track_id(self) -> str: ...
    @property
    def start(self) -> datetime.datetime: ...
    @property
    def end(self) -> datetime.datetime: ...
    @property
    def duration(self) -> datetime.timedelta: ...
    @property
    def party_id(self) -> str: ...

class CustomActivity(BaseActivity):
    name: Optional[str]
    emoji: Optional[PartialEmoji]
    def __init__(
        self,
        name: Optional[str],
        *,
        emoji: Optional[Union[PartialEmoji, str, _PartialEmojiDict]] = ...,
        **extra: Any,
    ) -> None: ...
    @property
    def type(self) -> ActivityType: ...
    def to_dict(self) -> _CustomActivityDict: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...



from typing import List, Optional

from .asset import _VALID_STATIC_ICON_FORMATS, Asset
from .guild import Guild
from .team import Team
from .user import User

class AppInfo:
    id: int
    name: str
    description: Optional[str]
    icon: Optional[str]
    rpc_origins: Optional[List[str]]
    bot_public: bool
    bot_require_code_grant: bool
    owner: User
    team: Team
    summary: str
    verify_key: str
    guild_id: Optional[int]
    primary_sku_id: Optional[int]
    slug: Optional[str]
    cover_image: Optional[str]
    @property
    def icon_url(self) -> Asset: ...
    def icon_url_as(
        self, *, format: _VALID_STATIC_ICON_FORMATS = ..., size: int = ...
    ) -> Asset: ...
    @property
    def cover_image_url(self) -> Asset: ...
    def cover_image_url_as(
        self, *, format: _VALID_STATIC_ICON_FORMATS = ..., size: int = ...
    ) -> Asset: ...
    @property
    def guild(self) -> Optional[Guild]: ...



from typing import Any, Dict, List, Optional, TypeVar, Union

import discord

from .bot import Bot
from .cog import Cog
from .core import Command

_C = TypeVar('_C', bound=Context)

class Context(discord.abc.Messageable):
    message: discord.Message
    bot: Bot[Any]
    args: List[Any]
    kwargs: Dict[str, Any]
    prefix: str
    command: Command[Any]
    invoked_with: Optional[str]
    invoked_parents: List[str]
    invoked_subcommand: Optional[Command[Any]]
    subcommand_passed: Optional[str]
    command_failed: bool
    async def invoke(
        self: _C, __command: Command[_C], *args: Any, **kwargs: Any
    ) -> Any: ...
    async def reinvoke(
        self, *, call_hooks: bool = ..., restart: bool = ...
    ) -> None: ...
    @property
    def valid(self) -> bool: ...
    @property
    def cog(self: _C) -> Optional[Cog[_C]]: ...
    @discord.utils.cached_property
    def guild(self) -> Optional[discord.Guild]: ...
    @discord.utils.cached_property
    def channel(
        self,
    ) -> Union[discord.TextChannel, discord.DMChannel, discord.GroupChannel]: ...
    @discord.utils.cached_property
    def author(self) -> Union[discord.User, discord.Member]: ...
    @discord.utils.cached_property
    def me(self) -> Union[discord.Member, discord.ClientUser]: ...
    @property
    def voice_client(self) -> Optional[discord.VoiceProtocol]: ...
    async def send_help(
        self: _C, __entity: Optional[Union[Command[_C], Cog[_C], str]]
    ) -> Any: ...
    async def reply(
        self,
        content: Optional[object] = ...,
        *,
        tts: bool = ...,
        embed: Optional[discord.Embed] = ...,
        file: Optional[discord.File] = ...,
        files: Optional[List[discord.File]] = ...,
        delete_after: Optional[float] = ...,
        nonce: Optional[int] = ...,
        allowed_mentions: Optional[discord.AllowedMentions] = ...,
        reference: Optional[Union[discord.Message, discord.MessageReference]] = ...,
        mention_author: Optional[bool] = ...,
    ) -> discord.Message: ...



from typing import BinaryIO, Optional, Union

class File:
    fp: Union[str, BinaryIO]
    filename: Optional[str]
    spoiler: bool
    def __init__(
        self,
        fp: Union[str, BinaryIO],
        filename: Optional[str] = ...,
        *,
        spoiler: bool = ...,
    ) -> None: ...
    def reset(self, *, seek: bool = ...) -> None: ...
    def close(self) -> None: ...



import asyncio
import datetime
from typing import (
    Any,
    Callable,
    Coroutine,
    Generic,
    Iterable,
    List,
    Optional,
    Pattern,
    Sequence,
    Set,
    Type,
    TypeVar,
    Union,
    overload,
)
from typing_extensions import Final

from aiohttp.web import Request

from .guild import Guild
from .invite import Invite
from .permissions import Permissions
from .template import Template

_T = TypeVar('_T')
_U_co = TypeVar('_U_co', covariant=True)

_FuncType = Callable[..., Any]
_F = TypeVar('_F', bound=_FuncType)

DISCORD_EPOCH: Final[int]

class cached_property(Generic[_T, _U_co]):
    def __init__(self, function: Callable[[_T], _U_co]) -> None: ...
    @overload
    def __get__(self, instance: _T, owner: Type[_T]) -> _U_co: ...
    @overload
    def __get__(
        self, instance: None, owner: Type[_T]
    ) -> cached_property[_T, _U_co]: ...

class CachedSlotProperty(Generic[_T, _U_co]):
    def __init__(self, name: str, function: Callable[[_T], _U_co]) -> None: ...
    @overload
    def __get__(self, instance: _T, owner: Type[_T]) -> _U_co: ...
    @overload
    def __get__(
        self, instance: None, owner: Type[_T]
    ) -> CachedSlotProperty[_T, _U_co]: ...

def cached_slot_property(
    name: str,
) -> Callable[[Callable[[_T], _U_co]], CachedSlotProperty[_T, _U_co]]: ...

class SequenceProxy(Sequence[_T]):
    def __init__(self, proxied: Sequence[_T]) -> None: ...
    @overload
    def __getitem__(self, i: int) -> _T: ...
    @overload
    def __getitem__(self, s: slice) -> Sequence[_T]: ...
    def __len__(self) -> int: ...
    def index(self, value: _T, start: int = ..., end: int = ...) -> int: ...
    def count(self, value: _T) -> int: ...

def parse_time(timestamp: Optional[str]) -> Optional[datetime.datetime]: ...
def copy_doc(original: _F) -> Callable[[Callable[..., Any]], _F]: ...
def deprecated(instead: Optional[str] = ...) -> Callable[[_F], _F]: ...
def oauth_url(
    client_id: str,
    permissions: Optional[Permissions] = ...,
    guild: Optional[Guild] = ...,
    redirect_uri: Optional[str] = ...,
    scopes: Optional[Iterable[str]] = ...,
) -> str: ...
def snowflake_time(id: int) -> datetime.datetime: ...
def time_snowflake(datetime_obj: datetime.datetime, high: bool = ...) -> int: ...
def find(predicate: Callable[[_T], bool], seq: Iterable[_T]) -> Optional[_T]: ...
def get(iterable: Iterable[_T], **attrs: Any) -> Optional[_T]: ...
def _unique(iterable: Iterable[_T]) -> List[_T]: ...
def _get_as_snowflake(data: Any, key: str) -> Optional[int]: ...
def _get_mime_type_for_image(data: Union[bytes, bytearray]) -> str: ...
def _bytes_to_base64_data(data: Union[bytes, bytearray]) -> str: ...
def to_json(obj: Any) -> str: ...
def _parse_ratelimit_header(request: Request, *, use_clock: bool = ...) -> float: ...
async def maybe_coroutine(
    f: Callable[..., Union[_T, Coroutine[Any, Any, _T]]], *args: Any, **kwargs: Any
) -> _T: ...
async def async_all(
    gen: Iterable[Union[Any, Coroutine[Any, Any, Any]]],
    *,
    check: Callable[[Any], bool] = ...,
) -> bool: ...
async def sane_wait_for(
    futures: List[asyncio.Future[_T]], *, timeout: float
) -> Set[asyncio.Future[_T]]: ...
@overload
async def sleep_until(when: datetime.datetime) -> None: ...
@overload
async def sleep_until(when: datetime.datetime, result: _T) -> _T: ...
def valid_icon_size(size: int) -> bool: ...
def _string_width(string: str, *, _IS_ASCII: Pattern[str] = ...) -> int: ...
def resolve_invite(invite: Union[Invite, str]) -> str: ...
def resolve_template(code: Union[Template, str]) -> str: ...
def remove_markdown(text: str, *, ignore_links: bool = ...) -> str: ...
def escape_markdown(
    text: str, *, as_needed: bool = ..., ignore_links: bool = ...
) -> str: ...
def escape_mentions(text: str) -> str: ...



from typing import Any, Optional, Tuple, Type, TypeVar

_C = TypeVar('_C', bound=Colour)

class Colour:
    value: int
    def __init__(self, value: int) -> None: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...
    @property
    def r(self) -> int: ...
    @property
    def g(self) -> int: ...
    @property
    def b(self) -> int: ...
    def to_rgb(self) -> Tuple[int, int, int]: ...
    @classmethod
    def from_rgb(cls: Type[_C], r: int, g: int, b: int) -> _C: ...
    @classmethod
    def from_hsv(cls: Type[_C], h: float, s: float, v: float) -> _C: ...
    @classmethod
    def default(cls: Type[_C]) -> _C: ...
    @classmethod
    def random(cls: Type[_C], *, seed: Optional[Any] = ...) -> _C: ...
    @classmethod
    def teal(cls: Type[_C]) -> _C: ...
    @classmethod
    def dark_teal(cls: Type[_C]) -> _C: ...
    @classmethod
    def green(cls: Type[_C]) -> _C: ...
    @classmethod
    def dark_green(cls: Type[_C]) -> _C: ...
    @classmethod
    def blue(cls: Type[_C]) -> _C: ...
    @classmethod
    def dark_blue(cls: Type[_C]) -> _C: ...
    @classmethod
    def purple(cls: Type[_C]) -> _C: ...
    @classmethod
    def dark_purple(cls: Type[_C]) -> _C: ...
    @classmethod
    def magenta(cls: Type[_C]) -> _C: ...
    @classmethod
    def dark_magenta(cls: Type[_C]) -> _C: ...
    @classmethod
    def gold(cls: Type[_C]) -> _C: ...
    @classmethod
    def dark_gold(cls: Type[_C]) -> _C: ...
    @classmethod
    def orange(cls: Type[_C]) -> _C: ...
    @classmethod
    def dark_orange(cls: Type[_C]) -> _C: ...
    @classmethod
    def red(cls: Type[_C]) -> _C: ...
    @classmethod
    def dark_red(cls: Type[_C]) -> _C: ...
    @classmethod
    def lighter_grey(cls: Type[_C]) -> _C: ...
    @classmethod
    def lighter_gray(cls: Type[_C]) -> _C: ...
    @classmethod
    def dark_grey(cls: Type[_C]) -> _C: ...
    @classmethod
    def dark_gray(cls: Type[_C]) -> _C: ...
    @classmethod
    def light_grey(cls: Type[_C]) -> _C: ...
    @classmethod
    def light_gray(cls: Type[_C]) -> _C: ...
    @classmethod
    def darker_grey(cls: Type[_C]) -> _C: ...
    @classmethod
    def darker_gray(cls: Type[_C]) -> _C: ...
    @classmethod
    def blurple(cls: Type[_C]) -> _C: ...
    @classmethod
    def greyple(cls: Type[_C]) -> _C: ...
    @classmethod
    def dark_theme(cls: Type[_C]) -> _C: ...

Color = Colour



import asyncio
import threading
from typing import (
    Any,
    Callable,
    ClassVar,
    Dict,
    Iterable,
    List,
    NamedTuple,
    Optional,
    Type,
    TypeVar,
    Union,
    type_check_only,
)
from typing_extensions import Literal, TypedDict

import aiohttp

from .activity import BaseActivity
from .client import Client
from .enums import SpeakingState
from .voice_client import VoiceClient

# TODO: remove this comment when a new version of black comes out
@type_check_only
class _KeepAlivePayloadDict(TypedDict):
    op: int
    d: int

class ReconnectWebSocket(Exception):
    shard_id: int
    resume: bool
    op: Literal['RESUME', 'IDENTIFY']
    def __init__(self, shard_id: int, *, resume: bool = ...) -> None: ...

class WebSocketClosure(Exception): ...

class EventListener(NamedTuple):
    predicate: Callable[[Any], bool]
    event: str
    result: Optional[Callable[[Any], Any]]
    future: asyncio.Future[Any]

class GatewayRatelimiter:
    def __init__(self, count: int = ..., per: float = ...) -> None: ...
    def get_delay(self) -> float: ...
    def is_ratelimited(self) -> bool: ...
    async def block(self) -> None: ...

class KeepAliveHandler(threading.Thread):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def run(self) -> None: ...
    def get_payload(self) -> _KeepAlivePayloadDict: ...
    def stop(self) -> None: ...
    def tick(self) -> None: ...
    def ack(self) -> None: ...

class VoiceKeepAliveHandler(KeepAliveHandler): ...

_DWS = TypeVar('_DWS', bound=DiscordWebSocket)

class DiscordClientWebSocketResponse(aiohttp.ClientWebSocketResponse):
    async def close(self, *, code: int = ..., message: bytes = ...) -> bool: ...

class DiscordWebSocket:
    DISPATCH: ClassVar[int]
    HEARTBEAT: ClassVar[int]
    IDENTIFY: ClassVar[int]
    PRESENCE: ClassVar[int]
    VOICE_STATE: ClassVar[int]
    VOICE_PING: ClassVar[int]
    RESUME: ClassVar[int]
    RECONNECT: ClassVar[int]
    REQUEST_MEMBERS: ClassVar[int]
    INVALIDATE_SESSION: ClassVar[int]
    HELLO: ClassVar[int]
    HEARTBEAT_ACK: ClassVar[int]
    GUILD_SYNC: ClassVar[int]

    thread_id: int
    session_id: Optional[int]
    sequence: Optional[int]
    @property
    def open(self) -> bool: ...
    def is_ratelimited(self) -> bool: ...
    @classmethod
    async def from_client(
        cls: Type[_DWS],
        client: Client,
        *,
        initial: bool = ...,
        gateway: Optional[str] = ...,
        shard_id: Optional[int] = ...,
        session: Optional[int] = ...,
        sequence: Optional[int] = ...,
        resume: bool = ...,
    ) -> _DWS: ...
    def wait_for(
        self,
        event: str,
        predicate: Callable[[Any], bool],
        result: Optional[Callable[[Any], Any]] = ...,
    ) -> asyncio.Future[Any]: ...
    async def identify(self) -> None: ...
    async def resume(self) -> None: ...
    async def received_message(self, msg: Union[str, bytes]) -> None: ...
    @property
    def latency(self) -> float: ...
    async def poll_event(self) -> None: ...
    async def send(self, data: str) -> None: ...
    async def send_as_json(self, data: Any) -> None: ...
    async def send_heartbeat(self, data: Any) -> None: ...
    async def change_presence(
        self,
        *,
        activity: Optional[BaseActivity] = ...,
        status: Optional[str] = ...,
        afk: bool = ...,
        since: float = ...,
    ) -> None: ...
    async def request_sync(self, guild_ids: Iterable[int]) -> None: ...
    async def request_chunks(
        self,
        guild_id: int,
        query: Optional[str] = ...,
        *,
        limit: int,
        user_ids: Optional[List[int]] = ...,
        presences: bool = ...,
        nonce: Optional[str] = ...,
    ) -> None: ...
    async def voice_state(
        self,
        guild_id: int,
        channel_id: Optional[int],
        self_mute: bool = ...,
        self_deaf: bool = ...,
    ) -> None: ...
    async def close(self, code: int = ...) -> None: ...

_DVWS = TypeVar('_DVWS', bound=DiscordVoiceWebSocket)

class DiscordVoiceWebSocket:
    IDENTIFY: ClassVar[int]
    SELECT_PROTOCOL: ClassVar[int]
    READY: ClassVar[int]
    HEARTBEAT: ClassVar[int]
    SESSION_DESCRIPTION: ClassVar[int]
    SPEAKING: ClassVar[int]
    HEARTBEAT_ACK: ClassVar[int]
    RESUME: ClassVar[int]
    HELLO: ClassVar[int]
    RESUMED: ClassVar[int]
    CLIENT_CONNECT: ClassVar[int]
    CLIENT_DISCONNECT: ClassVar[int]

    thread_id: int
    async def send_as_json(self, data: Any) -> None: ...
    async def send_heartbeat(self, data: Any) -> None: ...
    async def resume(self) -> None: ...
    async def identify(self) -> None: ...
    @classmethod
    async def from_client(
        cls: Type[_DVWS], client: VoiceClient, *, resume: bool = ...
    ) -> _DVWS: ...
    async def select_protocol(self, ip: str, port: str, mode: str) -> None: ...
    async def client_connect(self) -> None: ...
    async def speak(self, state: SpeakingState = ...) -> None: ...
    async def received_message(self, msg: Dict[str, Any]) -> None: ...
    async def initial_connection(self, data: Dict[str, Any]) -> None: ...
    @property
    def latency(self) -> float: ...
    @property
    def average_latency(self) -> float: ...
    async def load_secret_key(self, data: Dict[str, Any]) -> None: ...
    async def poll_event(self) -> None: ...
    async def close(self, code: int = ...) -> None: ...



from enum import Enum as Enum
from typing import Optional
from typing_extensions import Literal

# Technically, these enumerations do not inherit from enum.Enum, but because type checking
# does something special for enum.Enum and discord.enums.Enum is a drop-in replacement for
# enum.Enum, typings are using enum.Enum directly

class ChannelType(Enum):
    text: int
    private: int
    voice: int
    group: int
    category: int
    news: int
    store: int
    stage_voice: int

class MessageType(Enum):
    default: int
    recipient_add: int
    recipient_remove: int
    call: int
    channel_name_change: int
    channel_icon_change: int
    pins_add: int
    new_member: int
    premium_guild_subscription: int
    premium_guild_tier_1: int
    premium_guild_tier_2: int
    premium_guild_tier_3: int
    channel_follow_add: int
    guild_stream: int
    guild_discovery_disqualified: int
    guild_discovery_requalified: int
    guild_discovery_grace_period_initial_warning: int
    guild_discovery_grace_period_final_warning: int

class VoiceRegion(Enum):
    us_west: str
    us_east: str
    us_south: str
    us_central: str
    eu_west: str
    eu_central: str
    singapore: str
    london: str
    sydney: str
    amsterdam: str
    frankfurt: str
    brazil: str
    hongkong: str
    russia: str
    japan: str
    southafrica: str
    south_korea: str
    india: str
    europe: str
    dubai: str
    vip_us_east: str
    vip_us_west: str
    vip_amsterdam: str

class SpeakingState(Enum):
    none: int
    voice: int
    soundshare: int
    priority: int
    def __int__(self) -> int: ...

class VerificationLevel(Enum):
    none: int
    low: int
    medium: int
    high: int
    table_flip: int
    extreme: int
    double_table_flip: int
    very_high: int

class ContentFilter(Enum):
    disabled: int
    no_role: int
    all_members: int

class UserContentFilter(Enum):
    disabled: int
    friends: int
    all_messages: int

class FriendFlags(Enum):
    noone: int
    mutual_guilds: int
    mutual_friends: int
    guild_and_friends: int
    everyone: int

class Theme(Enum):
    light: str
    dark: str

class Status(Enum):
    online: str
    offline: str
    idle: str
    dnd: str
    do_not_disturb: str
    invisible: str

class DefaultAvatar(Enum):
    blurple: int
    grey: int
    gray: int
    green: int
    orange: int
    red: int

class RelationshipType(Enum):
    friend: int
    blocked: int
    incoming_request: int
    outgoing_request: int

class NotificationLevel(Enum):
    all_messages: int
    only_mentions: int

class AuditLogActionCategory(Enum):
    create: int
    delete: int
    update: int

class AuditLogAction(Enum):
    guild_update: int
    channel_create: int
    channel_update: int
    channel_delete: int
    overwrite_create: int
    overwrite_update: int
    overwrite_delete: int
    kick: int
    member_prune: int
    ban: int
    unban: int
    member_update: int
    member_role_update: int
    member_move: int
    member_disconnect: int
    bot_add: int
    role_create: int
    role_update: int
    role_delete: int
    invite_create: int
    invite_update: int
    invite_delete: int
    webhook_create: int
    webhook_update: int
    webhook_delete: int
    emoji_create: int
    emoji_update: int
    emoji_delete: int
    message_delete: int
    message_bulk_delete: int
    message_pin: int
    message_unpin: int
    integration_create: int
    integration_update: int
    integration_delete: int
    @property
    def category(self) -> Optional[AuditLogActionCategory]: ...
    @property
    def target_type(
        self,
    ) -> Literal[
        'all',
        'guild',
        'channel',
        'user',
        'role',
        'invite',
        'webhook',
        'emoji',
        'message',
        'integration',
    ]: ...

class UserFlags(Enum):
    staff: int
    partner: int
    hypesquad: int
    bug_hunter: int
    mfa_sms: int
    premium_promo_dismissed: int
    hypesquad_bravery: int
    hypesquad_brilliance: int
    hypesquad_balance: int
    early_supporter: int
    team_user: int
    system: int
    has_unread_urgent_messages: int
    bug_hunter_level_2: int
    verified_bot: int
    verified_bot_developer: int

class ActivityType(Enum):
    unknown: int
    playing: int
    streaming: int
    listening: int
    watching: int
    custom: int
    competing: int
    def __int__(self) -> int: ...

class HypeSquadHouse(Enum):
    bravery: int
    brilliance: int
    balance: int

class PremiumType(Enum):
    nitro_classic: int
    nitro: int

class TeamMembershipState(Enum):
    invited: int
    accepted: int

class WebhookType(Enum):
    incoming: int
    channel_follower: int

class ExpireBehaviour(Enum):
    remove_role: int
    kick: int

ExpireBehavior = ExpireBehaviour

class StickerType(Enum):
    png: int
    apng: int
    lottie: int



from inspect import Parameter
from typing import (
    Any,
    Awaitable,
    Callable,
    ClassVar,
    Dict,
    Generic,
    Iterable,
    List,
    Mapping,
    Optional,
    Pattern,
    Sequence,
    TypeVar,
    Union,
    type_check_only,
)
from typing_extensions import Protocol, TypedDict

import discord

from .cog import Cog
from .context import Context
from .cooldowns import Cooldown
from .core import Command, Group, _CheckPredicate
from .errors import CommandError

_T = TypeVar('_T')
_MaybeAwaitable = Union[Awaitable[_T], _T]
_CT = TypeVar('_CT', bound=Context)
_HC = TypeVar('_HC', bound=HelpCommand[Any])

# TODO: remove this comment when a new version of black comes out
@type_check_only
class _CommandAttrs(TypedDict, total=False):
    name: str
    enabled: bool
    help: Optional[str]
    brief: Optional[str]
    usage: Optional[str]
    aliases: List[str]
    description: str
    hidden: bool
    rest_is_raw: bool
    ignore_extra: bool
    cooldown: Cooldown
    parent: Command[Context]
    checks: List[_CheckPredicate[Any]]

@type_check_only
class _PaginatorProtocol(Protocol):
    prefix: Optional[str]
    suffix: Optional[str]
    max_size: int
    linesep: str
    def clear(self) -> None: ...
    def add_line(self, line: str = ..., *, empty: bool = ...) -> None: ...
    def close_page(self) -> None: ...
    def __len__(self) -> int: ...
    @property
    def pages(self) -> List[str]: ...

class Paginator:
    prefix: Optional[str]
    suffix: Optional[str]
    max_size: int
    linesep: str
    def __init__(
        self,
        prefix: Optional[str] = ...,
        suffix: Optional[str] = ...,
        max_size: int = ...,
        linesep: str = ...,
    ) -> None: ...
    def clear(self) -> None: ...
    def add_line(self, line: str = ..., *, empty: bool = ...) -> None: ...
    def close_page(self) -> None: ...
    def __len__(self) -> int: ...
    @property
    def pages(self) -> List[str]: ...

class _HelpCommandImpl(Command[_CT]):
    async def prepare(self, ctx: _CT) -> None: ...
    @property
    def clean_params(self) -> Mapping[str, Parameter]: ...

class HelpCommand(Generic[_CT]):
    context: Optional[_CT]
    show_hidden: bool
    verify_checks: Optional[bool]
    command_attrs: _CommandAttrs
    cog: Optional[Cog[_CT]]

    MENTION_TRANSFORMS: ClassVar[Dict[str, str]]
    MENTION_PATTERN: ClassVar[Pattern[str]]
    def __init__(
        self,
        *,
        show_hidden: bool = ...,
        verify_checks: Optional[bool] = ...,
        command_attrs: _CommandAttrs = ...,
    ) -> None: ...
    def copy(self: _HC) -> _HC: ...
    def add_check(self, func: _CheckPredicate[_CT]) -> None: ...
    def remove_check(self, func: _CheckPredicate[_CT]) -> None: ...
    def get_bot_mapping(self) -> Dict[Optional[Cog[_CT]], List[Command[_CT]]]: ...
    @property
    def clean_prefix(self) -> str: ...
    @property
    def invoked_with(self) -> str: ...
    def get_command_signature(self, command: Command[_CT]) -> str: ...
    def remove_mentions(self, string: str) -> str: ...
    def command_not_found(self, string: str) -> _MaybeAwaitable[str]: ...
    def subcommand_not_found(
        self, command: Command[_CT], string: str
    ) -> _MaybeAwaitable[str]: ...
    async def filter_commands(
        self,
        commands: Iterable[Command[_CT]],
        *,
        sort: bool = ...,
        key: Optional[Callable[[Command[_CT]], Any]] = ...,
    ) -> List[Command[_CT]]: ...
    def get_max_size(self, commands: Sequence[Command[_CT]]) -> int: ...
    def get_destination(
        self,
    ) -> Union[discord.TextChannel, discord.DMChannel, discord.GroupChannel]: ...
    async def send_error_message(self, error: str) -> None: ...
    async def on_help_command_error(self, ctx: _CT, error: CommandError) -> None: ...
    async def send_bot_help(
        self, mapping: Mapping[Optional[Cog[_CT]], List[Command[_CT]]]
    ) -> Any: ...
    async def send_cog_help(self, cog: Cog[_CT]) -> Any: ...
    async def send_group_help(self, group: Group[_CT]) -> Any: ...
    async def send_command_help(self, command: Command[_CT]) -> Any: ...
    async def prepare_help_command(
        self, ctx: _CT, command: Optional[str] = ...
    ) -> None: ...
    async def command_callback(
        self, ctx: _CT, *, command: Optional[str] = ...
    ) -> Any: ...

class DefaultHelpCommand(HelpCommand[_CT]):
    width: int
    sort_commands: bool
    indent: int
    commands_heading: str
    no_category: str
    paginator: _PaginatorProtocol
    def __init__(
        self,
        *,
        show_hidden: bool = ...,
        verify_checks: bool = ...,
        command_attrs: _CommandAttrs = ...,
        width: int = ...,
        indent: int = ...,
        sort_commands: bool = ...,
        dm_help: Optional[bool] = ...,
        dm_help_threshold: Optional[int] = ...,
        commands_heading: str = ...,
        no_category: str = ...,
        paginator: Optional[_PaginatorProtocol] = ...,
    ) -> None: ...
    def shorten_text(self, text: str) -> str: ...
    def get_ending_note(self) -> str: ...
    def add_indented_commands(
        self,
        commands: Sequence[Command[_CT]],
        *,
        heading: str,
        max_size: Optional[int] = ...,
    ) -> None: ...
    async def send_pages(self) -> None: ...
    def add_command_formatting(self, command: Command[_CT]) -> None: ...
    async def prepare_help_command(self, ctx: _CT, command: Optional[str]) -> None: ...  # type: ignore[override]
    async def send_bot_help(
        self, mapping: Mapping[Optional[Cog[_CT]], List[Command[_CT]]]
    ) -> None: ...
    async def send_command_help(self, command: Command[_CT]) -> None: ...
    async def send_group_help(self, group: Group[_CT]) -> None: ...
    async def send_cog_help(self, cog: Cog[_CT]) -> None: ...

class MinimalHelpCommand(HelpCommand[_CT]):
    sort_commands: bool
    commands_heading: str
    aliases_heading: str
    no_category: str
    paginator: _PaginatorProtocol
    def __init__(
        self,
        *,
        show_hidden: bool = ...,
        verify_checks: bool = ...,
        command_attrs: _CommandAttrs = ...,
        sort_commands: bool = ...,
        commands_heading: str = ...,
        dm_help: Optional[bool] = ...,
        dm_help_threshold: Optional[int] = ...,
        aliases_heading: str = ...,
        no_category: str = ...,
        paginator: Optional[_PaginatorProtocol] = ...,
    ) -> None: ...
    async def send_pages(self) -> None: ...
    def get_opening_note(self) -> str: ...
    def get_command_signature(self, command: Command[_CT]) -> str: ...
    def get_ending_note(self) -> Optional[str]: ...
    def add_bot_commands_formatting(
        self, commands: Sequence[Command[_CT]], heading: str
    ) -> None: ...
    def add_subcommand_formatting(self, command: Command[_CT]) -> None: ...
    def add_aliases_formatting(self, aliases: Sequence[str]) -> None: ...
    def add_command_formatting(self, command: Command[_CT]) -> None: ...
    async def prepare_help_command(self, ctx: _CT, command: Optional[str]) -> None: ...  # type: ignore[override]
    async def send_bot_help(
        self, mapping: Mapping[Optional[Cog[_CT]], List[Command[_CT]]]
    ) -> None: ...
    async def send_cog_help(self, cog: Cog[_CT]) -> None: ...
    async def send_group_help(self, group: Group[_CT]) -> None: ...
    async def send_command_help(self, command: Command[_CT]) -> None: ...



import datetime
from typing import Any, Callable, ClassVar, Dict, Iterator, Optional, Tuple

from . import enums
from .abc import User as _ABCUser
from .guild import Guild
from .mixins import Hashable
from .utils import cached_property

class AuditLogDiff:
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[Tuple[str, Any]]: ...

class AuditLogChanges:
    TRANSFORMERS: ClassVar[
        Dict[str, Tuple[Optional[str], Optional[Callable[[Any, Any], AuditLogDiff]]]]
    ]
    before: AuditLogDiff
    after: AuditLogDiff

class AuditLogEntry(Hashable):
    id: int
    guild: Guild
    action: enums.AuditLogAction
    user: _ABCUser
    reason: Optional[str]
    extra: Any
    @cached_property
    def created_at(self) -> datetime.datetime: ...
    @cached_property
    def target(self) -> Any: ...
    @cached_property
    def category(self) -> Optional[enums.AuditLogActionCategory]: ...
    @cached_property
    def changes(self) -> AuditLogChanges: ...
    @cached_property
    def before(self) -> AuditLogDiff: ...
    @cached_property
    def after(self) -> AuditLogDiff: ...



import datetime
from typing import Any, Iterator, List, Optional, Tuple

from .asset import _VALID_ANIMATED_ICON_FORMATS, _VALID_STATIC_ICON_FORMATS, Asset
from .guild import Guild
from .partial_emoji import _EmojiTag
from .role import Role
from .user import User

class Emoji(_EmojiTag):
    name: str
    id: int
    require_colons: bool
    animated: bool
    managed: bool
    guild_id: int
    available: bool
    user: Optional[User]
    def __iter__(self) -> Iterator[Tuple[str, Any]]: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...
    @property
    def created_at(self) -> datetime.datetime: ...
    @property
    def url(self) -> Asset: ...
    @property
    def roles(self) -> List[Role]: ...
    @property
    def guild(self) -> Guild: ...
    def url_as(
        self,
        *,
        format: Optional[_VALID_ANIMATED_ICON_FORMATS] = ...,
        static_format: _VALID_STATIC_ICON_FORMATS = ...,
    ) -> Asset: ...
    def is_usable(self) -> bool: ...
    async def delete(self, *, reason: Optional[str] = ...) -> None: ...
    async def edit(
        self,
        *,
        name: Optional[str] = ...,
        roles: Optional[List[Role]] = ...,
        reason: Optional[str] = ...,
    ) -> None: ...



import asyncio
from typing import Any, Optional

from .abc import Messageable

class Typing:
    loop: asyncio.AbstractEventLoop
    messageable: Messageable
    task: Optional[asyncio.Task[None]]
    def __init__(self, messageable: Messageable) -> None: ...
    async def do_typing(self) -> None: ...
    def __enter__(self) -> Typing: ...
    def __exit__(self, exc_type: Any, exc: Any, tb: Any) -> None: ...
    async def __aenter__(self) -> Typing: ...
    async def __aexit__(self, exc_type: Any, exc: Any, tb: Any) -> None: ...



import sys
from typing import Any, BinaryIO, ClassVar, FrozenSet, Union
from typing_extensions import Final, Literal

if sys.version_info >= (3, 6):
    from os import PathLike

_VALID_STATIC_ICON_FORMATS = Literal['jpeg', 'jpg', 'webp', 'png']
_VALID_ANIMATED_ICON_FORMATS = Literal[_VALID_STATIC_ICON_FORMATS, 'gif']

VALID_STATIC_FORMATS: Final[FrozenSet[_VALID_STATIC_ICON_FORMATS]]
VALID_AVATAR_FORMATS: Final[FrozenSet[_VALID_ANIMATED_ICON_FORMATS]]

class Asset:
    BASE: ClassVar[str]
    def __len__(self) -> int: ...
    def __bool__(self) -> bool: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...
    async def read(self) -> bytes: ...
    if sys.version_info >= (3, 6):
        async def save(
            self, fp: Union[BinaryIO, PathLike[str], str], *, seek_begin: bool = ...
        ) -> int: ...
    else:
        async def save(
            self, fp: Union[BinaryIO, str], *, seek_begin: bool = ...
        ) -> int: ...



from typing import Any, ClassVar, Dict, Iterator, Optional, Set, Tuple, Type, TypeVar

from .flags import BaseFlags, alias_flag_value, flag_value

_P = TypeVar('_P', bound=Permissions)

class permission_alias(alias_flag_value):
    alias: str

class Permissions(BaseFlags):
    VALID_FLAGS: ClassVar[Dict[str, int]]

    create_instant_invite: flag_value
    kick_members: flag_value
    ban_members: flag_value
    administrator: flag_value
    manage_channels: flag_value
    manage_guild: flag_value
    add_reactions: flag_value
    view_audit_log: flag_value
    priority_speaker: flag_value
    stream: flag_value
    read_messages: flag_value
    view_channel: permission_alias
    send_messages: flag_value
    send_tts_messages: flag_value
    manage_messages: flag_value
    embed_links: flag_value
    attach_files: flag_value
    read_message_history: flag_value
    mention_everyone: flag_value
    external_emojis: flag_value
    use_external_emojis: permission_alias
    view_guild_insights: flag_value
    connect: flag_value
    speak: flag_value
    mute_members: flag_value
    deafen_members: flag_value
    move_members: flag_value
    use_voice_activation: flag_value
    change_nickname: flag_value
    manage_nicknames: flag_value
    manage_roles: flag_value
    manage_permissions: permission_alias
    manage_webhooks: flag_value
    manage_emojis: flag_value
    use_slash_commands: flag_value
    request_to_speak: flag_value
    def __init__(
        self,
        permissions: int = ...,
        *,
        create_instant_invite: bool = ...,
        kick_members: bool = ...,
        ban_members: bool = ...,
        administrator: bool = ...,
        manage_channels: bool = ...,
        manage_guild: bool = ...,
        add_reactions: bool = ...,
        view_audit_log: bool = ...,
        priority_speaker: bool = ...,
        stream: bool = ...,
        read_messages: bool = ...,
        view_channel: bool = ...,
        send_messages: bool = ...,
        send_tts_messages: bool = ...,
        manage_messages: bool = ...,
        embed_links: bool = ...,
        attach_files: bool = ...,
        read_message_history: bool = ...,
        mention_everyone: bool = ...,
        external_emojis: bool = ...,
        use_external_emojis: bool = ...,
        view_guild_insights: bool = ...,
        connect: bool = ...,
        speak: bool = ...,
        mute_members: bool = ...,
        deafen_members: bool = ...,
        move_members: bool = ...,
        use_voice_activation: bool = ...,
        change_nickname: bool = ...,
        manage_nicknames: bool = ...,
        manage_roles: bool = ...,
        manage_permissions: bool = ...,
        manage_webhooks: bool = ...,
        manage_emojis: bool = ...,
        use_slash_commands: bool = ...,
        request_to_speak: bool = ...,
    ) -> None: ...
    def is_subset(self, other: Any) -> bool: ...
    def is_superset(self, other: Any) -> bool: ...
    def is_strict_subset(self, other: Any) -> bool: ...
    def is_strict_superset(self, other: Any) -> bool: ...
    def __le__(self, other: Any) -> bool: ...
    def __ge__(self, other: Any) -> bool: ...
    def __lt__(self, other: Any) -> bool: ...
    def __gt__(self, other: Any) -> bool: ...
    @classmethod
    def none(cls: Type[_P]) -> _P: ...
    @classmethod
    def all(cls: Type[_P]) -> _P: ...
    @classmethod
    def all_channel(cls: Type[_P]) -> _P: ...
    @classmethod
    def general(cls: Type[_P]) -> _P: ...
    @classmethod
    def membership(cls: Type[_P]) -> _P: ...
    @classmethod
    def text(cls: Type[_P]) -> _P: ...
    @classmethod
    def voice(cls: Type[_P]) -> _P: ...
    @classmethod
    def stage(cls: Type[_P]) -> _P: ...
    @classmethod
    def advanced(cls: Type[_P]) -> _P: ...
    @classmethod
    def stage_moderator(cls: Type[_P]) -> _P: ...
    def update(
        self,
        *,
        create_instant_invite: bool = ...,
        kick_members: bool = ...,
        ban_members: bool = ...,
        administrator: bool = ...,
        manage_channels: bool = ...,
        manage_guild: bool = ...,
        add_reactions: bool = ...,
        view_audit_log: bool = ...,
        priority_speaker: bool = ...,
        stream: bool = ...,
        read_messages: bool = ...,
        view_channel: bool = ...,
        send_messages: bool = ...,
        send_tts_messages: bool = ...,
        manage_messages: bool = ...,
        embed_links: bool = ...,
        attach_files: bool = ...,
        read_message_history: bool = ...,
        mention_everyone: bool = ...,
        external_emojis: bool = ...,
        use_external_emojis: bool = ...,
        view_guild_insights: bool = ...,
        connect: bool = ...,
        speak: bool = ...,
        mute_members: bool = ...,
        deafen_members: bool = ...,
        move_members: bool = ...,
        use_voice_activation: bool = ...,
        change_nickname: bool = ...,
        manage_nicknames: bool = ...,
        manage_roles: bool = ...,
        manage_permissions: bool = ...,
        manage_webhooks: bool = ...,
        manage_emojis: bool = ...,
        use_slash_commands: bool = ...,
        request_to_speak: bool = ...,
    ) -> None: ...
    def handle_overwrite(self, allow: int, deny: int) -> None: ...

_PO = TypeVar('_PO', bound=PermissionOverwrite)

class PermissionOverwrite:
    VALID_NAMES: ClassVar[Set[str]]
    PURE_FLAGS: ClassVar[Set[str]]

    create_instant_invite: Optional[bool]
    kick_members: Optional[bool]
    ban_members: Optional[bool]
    administrator: Optional[bool]
    manage_channels: Optional[bool]
    manage_guild: Optional[bool]
    add_reactions: Optional[bool]
    view_audit_log: Optional[bool]
    priority_speaker: Optional[bool]
    stream: Optional[bool]
    read_messages: Optional[bool]
    view_channel: Optional[bool]
    send_messages: Optional[bool]
    send_tts_messages: Optional[bool]
    manage_messages: Optional[bool]
    embed_links: Optional[bool]
    attach_files: Optional[bool]
    read_message_history: Optional[bool]
    mention_everyone: Optional[bool]
    external_emojis: Optional[bool]
    use_external_emojis: Optional[bool]
    view_guild_insights: Optional[bool]
    connect: Optional[bool]
    speak: Optional[bool]
    mute_members: Optional[bool]
    deafen_members: Optional[bool]
    move_members: Optional[bool]
    use_voice_activation: Optional[bool]
    change_nickname: Optional[bool]
    manage_nicknames: Optional[bool]
    manage_roles: Optional[bool]
    manage_permissions: Optional[bool]
    manage_webhooks: Optional[bool]
    manage_emojis: Optional[bool]
    use_slash_commands: Optional[bool]
    request_to_speak: Optional[bool]
    def __init__(
        self,
        *,
        create_instant_invite: Optional[bool] = ...,
        kick_members: Optional[bool] = ...,
        ban_members: Optional[bool] = ...,
        administrator: Optional[bool] = ...,
        manage_channels: Optional[bool] = ...,
        manage_guild: Optional[bool] = ...,
        add_reactions: Optional[bool] = ...,
        view_audit_log: Optional[bool] = ...,
        priority_speaker: Optional[bool] = ...,
        stream: Optional[bool] = ...,
        read_messages: Optional[bool] = ...,
        view_channel: Optional[bool] = ...,
        send_messages: Optional[bool] = ...,
        send_tts_messages: Optional[bool] = ...,
        manage_messages: Optional[bool] = ...,
        embed_links: Optional[bool] = ...,
        attach_files: Optional[bool] = ...,
        read_message_history: Optional[bool] = ...,
        mention_everyone: Optional[bool] = ...,
        external_emojis: Optional[bool] = ...,
        use_external_emojis: Optional[bool] = ...,
        view_guild_insights: Optional[bool] = ...,
        connect: Optional[bool] = ...,
        speak: Optional[bool] = ...,
        mute_members: Optional[bool] = ...,
        deafen_members: Optional[bool] = ...,
        move_members: Optional[bool] = ...,
        use_voice_activation: Optional[bool] = ...,
        change_nickname: Optional[bool] = ...,
        manage_nicknames: Optional[bool] = ...,
        manage_roles: Optional[bool] = ...,
        manage_permissions: Optional[bool] = ...,
        manage_webhooks: Optional[bool] = ...,
        manage_emojis: Optional[bool] = ...,
        use_slash_commands: Optional[bool] = ...,
        request_to_speak: Optional[bool] = ...,
    ) -> None: ...
    def __eq__(self, other: Any) -> bool: ...
    def pair(self) -> Tuple[Permissions, Permissions]: ...
    @classmethod
    def from_pair(cls: Type[_PO], allow: Permissions, deny: Permissions) -> _PO: ...
    def is_empty(self) -> bool: ...
    def update(
        self,
        *,
        create_instant_invite: Optional[bool] = ...,
        kick_members: Optional[bool] = ...,
        ban_members: Optional[bool] = ...,
        administrator: Optional[bool] = ...,
        manage_channels: Optional[bool] = ...,
        manage_guild: Optional[bool] = ...,
        add_reactions: Optional[bool] = ...,
        view_audit_log: Optional[bool] = ...,
        priority_speaker: Optional[bool] = ...,
        stream: Optional[bool] = ...,
        read_messages: Optional[bool] = ...,
        view_channel: Optional[bool] = ...,
        send_messages: Optional[bool] = ...,
        send_tts_messages: Optional[bool] = ...,
        manage_messages: Optional[bool] = ...,
        embed_links: Optional[bool] = ...,
        attach_files: Optional[bool] = ...,
        read_message_history: Optional[bool] = ...,
        mention_everyone: Optional[bool] = ...,
        external_emojis: Optional[bool] = ...,
        use_external_emojis: Optional[bool] = ...,
        view_guild_insights: Optional[bool] = ...,
        connect: Optional[bool] = ...,
        speak: Optional[bool] = ...,
        mute_members: Optional[bool] = ...,
        deafen_members: Optional[bool] = ...,
        move_members: Optional[bool] = ...,
        use_voice_activation: Optional[bool] = ...,
        change_nickname: Optional[bool] = ...,
        manage_nicknames: Optional[bool] = ...,
        manage_roles: Optional[bool] = ...,
        manage_permissions: Optional[bool] = ...,
        manage_webhooks: Optional[bool] = ...,
        manage_emojis: Optional[bool] = ...,
        use_slash_commands: Optional[bool] = ...,
        request_to_speak: Optional[bool] = ...,
    ) -> None: ...
    def __iter__(self) -> Iterator[Tuple[str, Optional[bool]]]: ...



import datetime
from typing import (
    Any,
    Dict,
    Iterable,
    List,
    NamedTuple,
    Optional,
    Tuple,
    Union,
    overload,
)
from typing_extensions import Literal

from .abc import Snowflake
from .asset import _VALID_ANIMATED_ICON_FORMATS as _VALID_ANIMATED_ICON_FORMATS
from .asset import _VALID_STATIC_ICON_FORMATS as _VALID_STATIC_ICON_FORMATS
from .asset import Asset
from .channel import (
    CategoryChannel,
    StageChannel,
    StoreChannel,
    TextChannel,
    VoiceChannel,
)
from .colour import Colour
from .emoji import Emoji
from .enums import (
    AuditLogAction,
    ContentFilter,
    NotificationLevel,
    VerificationLevel,
    VoiceRegion,
)
from .flags import SystemChannelFlags
from .integrations import Integration
from .invite import Invite
from .iterators import AuditLogIterator, MemberIterator
from .member import Member
from .mixins import Hashable
from .permissions import PermissionOverwrite, Permissions
from .role import Role
from .template import Template
from .user import User
from .voice_client import VoiceProtocol
from .webhook import _AsyncWebhook
from .widget import Widget

_VALID_FEATURES = Literal[
    'VIP_REGIONS',
    'VANITY_URL',
    'INVITE_SPLASH',
    'VERIFIED',
    'PARTNERED',
    'MORE_EMOJI',
    'DISCOVERABLE',
    'FEATURABLE',
    'COMMUNITY',
    'COMMERCE',
    'PUBLIC',
    'NEWS',
    'BANNER',
    'ANIMATED_ICON',
    'PUBLIC_DISABLED',
    'WELCOME_SCREEN_ENABLED',
    'MEMBER_VERIFICATION_GATE_ENABLED',
    'PREVIEW_ENABLED',
]

class BanEntry(NamedTuple):
    reason: str
    user: User

class Guild(Hashable):
    name: str
    emojis: Tuple[Emoji, ...]
    region: VoiceRegion
    afk_timeout: int
    afk_channel: Optional[VoiceChannel]
    icon: Optional[str]
    id: int
    owner_id: int
    unavailable: bool
    max_presences: Optional[int]
    max_members: Optional[int]
    max_video_channel_users: Optional[int]
    banner: Optional[str]
    description: Optional[str]
    mfa_level: int
    verification_level: VerificationLevel
    default_notifications: NotificationLevel
    explicit_content_filter: ContentFilter
    features: List[_VALID_FEATURES]
    splash: Optional[str]
    premium_tier: int
    premium_subscription_count: int
    preferred_locale: Optional[str]
    discovery_splash: str
    @property
    def channels(
        self,
    ) -> List[Union[TextChannel, VoiceChannel, CategoryChannel, StoreChannel]]: ...
    @property
    def large(self) -> bool: ...
    @property
    def voice_channels(self) -> List[VoiceChannel]: ...
    @property
    def me(self) -> Member: ...
    @property
    def voice_client(self) -> Optional[VoiceProtocol]: ...
    @property
    def text_channels(self) -> List[TextChannel]: ...
    @property
    def stage_channels(self) -> List[StageChannel]: ...
    @property
    def categories(self) -> List[CategoryChannel]: ...
    def by_category(
        self,
    ) -> List[
        Tuple[
            Optional[CategoryChannel],
            List[Union[TextChannel, VoiceChannel, StoreChannel]],
        ]
    ]: ...
    def get_channel(
        self, channel_id: int
    ) -> Optional[Union[TextChannel, VoiceChannel, StoreChannel, CategoryChannel]]: ...
    @property
    def system_channel(self) -> Optional[TextChannel]: ...
    @property
    def system_channel_flags(self) -> SystemChannelFlags: ...
    @property
    def rules_channel(self) -> Optional[TextChannel]: ...
    @property
    def public_updates_channel(self) -> Optional[TextChannel]: ...
    @property
    def emoji_limit(self) -> int: ...
    @property
    def bitrate_limit(self) -> float: ...
    @property
    def filesize_limit(self) -> int: ...
    @property
    def members(self) -> List[Member]: ...
    def get_member(self, user_id: int) -> Optional[Member]: ...
    @property
    def premium_subscribers(self) -> List[Member]: ...
    @property
    def roles(self) -> List[Role]: ...
    def get_role(self, role_id: int) -> Optional[Role]: ...
    @property
    def default_role(self) -> Role: ...
    @property
    def premium_subscriber_role(self) -> Optional[Role]: ...
    @property
    def self_role(self) -> Optional[Role]: ...
    @property
    def owner(self) -> Optional[Member]: ...
    @property
    def icon_url(self) -> Asset: ...
    def is_icon_animated(self) -> bool: ...
    def icon_url_as(
        self,
        *,
        format: Optional[_VALID_ANIMATED_ICON_FORMATS] = ...,
        static_format: _VALID_STATIC_ICON_FORMATS = ...,
        size: int = ...,
    ) -> Asset: ...
    @property
    def banner_url(self) -> Asset: ...
    def banner_url_as(
        self, *, format: _VALID_STATIC_ICON_FORMATS = ..., size: int = ...
    ) -> Asset: ...
    @property
    def splash_url(self) -> Asset: ...
    def splash_url_as(
        self, *, format: _VALID_STATIC_ICON_FORMATS = ..., size: int = ...
    ) -> Asset: ...
    @property
    def discovery_splash_url(self) -> Asset: ...
    def discovery_splash_url_as(
        self, *, format: _VALID_STATIC_ICON_FORMATS = ..., size: int = ...
    ) -> Asset: ...
    @property
    def member_count(self) -> int: ...
    @property
    def chunked(self) -> bool: ...
    @property
    def shard_id(self) -> Optional[int]: ...
    @property
    def created_at(self) -> datetime.datetime: ...
    def get_member_named(self, name: str) -> Optional[Member]: ...
    async def create_text_channel(
        self,
        name: str,
        *,
        overwrites: Optional[Dict[Union[Role, Member], PermissionOverwrite]] = ...,
        category: Optional[CategoryChannel] = ...,
        position: int = ...,
        topic: str = ...,
        slowmode_delay: int = ...,
        nsfw: bool = ...,
        reason: Optional[str] = ...,
    ) -> TextChannel: ...
    async def create_voice_channel(
        self,
        name: str,
        *,
        overwrites: Optional[Dict[Union[Role, Member], PermissionOverwrite]] = ...,
        category: Optional[CategoryChannel] = ...,
        bitrate: int = ...,
        position: int = ...,
        user_limit: int = ...,
        rtc_region: Optional[VoiceRegion] = ...,
        reason: Optional[str] = ...,
    ) -> VoiceChannel: ...
    async def create_stage_channel(
        self,
        name: str,
        *,
        topic: str = ...,
        overwrites: Optional[Dict[Union[Role, Member], PermissionOverwrite]] = ...,
        category: Optional[CategoryChannel] = ...,
        position: int = ...,
        reason: Optional[str] = ...,
    ) -> StageChannel: ...
    async def create_category(
        self,
        name: str,
        *,
        overwrites: Optional[Dict[Union[Role, Member], PermissionOverwrite]] = ...,
        position: Optional[int] = ...,
        reason: Optional[str] = ...,
    ) -> CategoryChannel: ...
    async def create_category_channel(
        self,
        name: str,
        *,
        overwrites: Optional[Dict[Union[Role, Member], PermissionOverwrite]] = ...,
        position: Optional[int] = ...,
        reason: Optional[str] = ...,
    ) -> CategoryChannel: ...
    async def leave(self) -> None: ...
    async def delete(self) -> None: ...
    async def edit(
        self,
        *,
        reason: Optional[str] = ...,
        name: str = ...,
        description: str = ...,
        icon: Optional[bytes] = ...,
        splash: Optional[bytes] = ...,
        banner: Optional[bytes] = ...,
        region: VoiceRegion = ...,
        afk_channel: Optional[VoiceChannel] = ...,
        afk_timeout: int = ...,
        owner: Member = ...,
        verification_level: VerificationLevel = ...,
        default_notifications: NotificationLevel = ...,
        explicit_content_filter: ContentFilter = ...,
        vanity_code: str = ...,
        system_channel: Optional[TextChannel] = ...,
        system_channel_flags: SystemChannelFlags = ...,
        preferred_locale: str = ...,
        rules_channel: Optional[TextChannel] = ...,
        public_updates_channel: Optional[TextChannel] = ...,
    ) -> None: ...
    async def fetch_channels(
        self,
    ) -> List[
        Union[TextChannel, VoiceChannel, StageChannel, StoreChannel, CategoryChannel]
    ]: ...
    def fetch_members(
        self,
        *,
        limit: Optional[int] = ...,
        after: Optional[Union[Snowflake, datetime.datetime]] = ...,
    ) -> MemberIterator: ...
    async def fetch_member(self, member_id: int) -> Member: ...
    async def fetch_ban(self, user: Snowflake) -> BanEntry: ...
    async def bans(self) -> List[BanEntry]: ...
    async def prune_members(
        self,
        *,
        days: int,
        compute_prune_count: bool = ...,
        roles: Optional[Iterable[Snowflake]] = ...,
        reason: Optional[str] = ...,
    ) -> Optional[int]: ...
    async def templates(self) -> List[Template]: ...
    async def webhooks(self) -> List[_AsyncWebhook]: ...
    async def estimate_pruned_members(
        self, *, days: int, roles: Optional[Snowflake] = ...
    ) -> int: ...
    async def invites(self) -> List[Invite]: ...
    async def create_template(
        self, *, name: str, description: Optional[str] = ...
    ) -> Template: ...
    async def fetch_emojis(self) -> List[Emoji]: ...
    async def fetch_emoji(self, emoji_id: int) -> Emoji: ...
    async def create_integration(self, *, type: str, id: int) -> None: ...
    async def integrations(self) -> List[Integration]: ...
    async def create_custom_emoji(
        self,
        *,
        name: str,
        image: Union[bytes, bytearray],
        roles: Optional[Iterable[Role]] = ...,
        reason: Optional[str] = ...,
    ) -> Emoji: ...
    async def fetch_roles(self) -> List[Role]: ...
    async def create_role(
        self,
        *,
        name: str,
        permissions: Permissions = ...,
        colour: Union[Colour, int] = ...,
        color: Union[Colour, int] = ...,
        hoist: bool = ...,
        mentionable: bool = ...,
        reason: Optional[str] = ...,
    ) -> Role: ...
    async def edit_role_positions(
        self, positions: Dict[Role, int], *, reason: Optional[str] = ...
    ) -> List[Role]: ...
    async def kick(self, user: Snowflake, *, reason: Optional[str] = ...) -> None: ...
    async def ban(
        self,
        user: Snowflake,
        *,
        reason: Optional[str] = ...,
        delete_message_days: int = ...,
    ) -> None: ...
    async def unban(self, user: Snowflake, *, reason: Optional[str] = ...) -> None: ...
    async def vanity_invite(self) -> Invite: ...
    def ack(self) -> Any: ...
    def audit_logs(
        self,
        *,
        limit: int = ...,
        before: Optional[Union[Snowflake, datetime.datetime]] = ...,
        after: Optional[Union[Snowflake, datetime.datetime]] = ...,
        oldest_first: Optional[bool] = ...,
        user: Optional[Snowflake] = ...,
        action: Optional[AuditLogAction] = ...,
    ) -> AuditLogIterator: ...
    async def widget(self) -> Widget: ...
    async def chunk(self, *, cache: bool = ...) -> List[Member]: ...
    @overload
    async def query_members(
        self, query: str, *, limit: int = ..., presences: bool = ..., cache: bool = ...
    ) -> List[Member]: ...
    @overload
    async def query_members(
        self,
        query: None = ...,
        *,
        limit: int = ...,
        user_ids: Optional[List[int]] = ...,
        presences: bool = ...,
        cache: bool = ...,
    ) -> List[Member]: ...
    async def change_voice_state(
        self,
        *,
        channel: Optional[VoiceChannel],
        self_mute: bool = ...,
        self_deaf: bool = ...,
    ) -> None: ...



from typing import BinaryIO, Iterator, Tuple

from .errors import DiscordException

class OggError(DiscordException): ...

class OggPage:
    segtable: bytes
    data: bytes
    def __init__(self, stream: BinaryIO) -> None: ...
    def iter_packets(self) -> Iterator[Tuple[bytes, bool]]: ...

class OggStream:
    stream: BinaryIO
    def __init__(self, stream: BinaryIO) -> None: ...
    def iter_packets(self) -> Iterator[bytes]: ...



import datetime
from builtins import _SupportsIndex
from typing import SupportsInt, Text, Union

from .mixins import Hashable

class Object(Hashable):
    id: int
    def __init__(self, id: Union[Text, bytes, SupportsInt, _SupportsIndex]) -> None: ...
    @property
    def created_at(self) -> datetime.datetime: ...
